@@ -2,9 +2,35 @@
 #include "computation/machine/args.H"
 #include "dp/2way.H"
 #include "imodel/imodel.H"
-#include "computation/expression/expression.H"
+#include "computation/expression/bool.H"
+#include "computation/expression/constructor.H"
 #include <boost/dynamic_bitset.hpp>
 #include "alignment/alignment.H"
+#include "sequence/sequence-format.H"
+#include "dp/2way.H"
+#include "models/site-compression.H"
+#include "util/cmdline.H"
+using std::string;
+using std::vector;
+
+using Alphabet = PtrBox<alphabet>;
+
+// #include "alignment/alignment-util.H"
+vector<int> alignment_row_counts(const alignment& A, int i, const vector<int>& counts);
+
+
+extern "C" closure builtin_function_flip_alignment(OperationArgs& Args)
+{
+    return Box<pairwise_alignment_t>(Args.evaluate(0).as_<Box<pairwise_alignment_t>>().flipped());
+}
+
+extern "C" closure builtin_function_unaligned_pairwise_alignment(OperationArgs& Args)
+{
+    int l1 = Args.evaluate(0).as_int();
+    int l2 = Args.evaluate(1).as_int();
+
+    return Box<pairwise_alignment_t>(make_unaligned_pairwise_alignment(l1,l2));
+}
 
 extern "C" closure builtin_function_pairwise_alignment_probability_from_counts(OperationArgs& Args)
 {
@@ -40,17 +66,22 @@ extern "C" closure builtin_function_pairwise_alignment_probability_from_counts(O
 
 extern "C" closure builtin_function_pairwise_alignment_length1(OperationArgs& Args)
 {
-    return {Args.evaluate(0).as_<pairwise_alignment_t>().length1()};
+    return {Args.evaluate(0).as_<Box<pairwise_alignment_t>>().length1()};
 }
 
 extern "C" closure builtin_function_pairwise_alignment_length2(OperationArgs& Args)
 {
-    return {Args.evaluate(0).as_<pairwise_alignment_t>().length2()};
+    return {Args.evaluate(0).as_<Box<pairwise_alignment_t>>().length2()};
+}
+
+extern "C" closure builtin_function_alignment_length(OperationArgs& Args)
+{
+    return {Args.evaluate(0).as_<Box<alignment>>().length()};
 }
 
 extern "C" closure builtin_function_transition_counts(OperationArgs& Args)
 {
-    const pairwise_alignment_t& A = Args.evaluate(0).as_<pairwise_alignment_t>();
+    const auto& A = Args.evaluate(0).as_<Box<pairwise_alignment_t>>().value();
 
     Box<matrix<int>> counts(5,5,0);
 
@@ -67,8 +98,6 @@ extern "C" closure builtin_function_transition_counts(OperationArgs& Args)
     return counts;
 }
 
-using std::vector;
-
 extern "C" closure builtin_function_rs07_lengthp(OperationArgs& Args)
 {
     double e = Args.evaluate(0).as_double();
@@ -274,7 +303,7 @@ extern "C" closure builtin_function_bitmask_from_alignment(OperationArgs& Args)
     using boost::dynamic_bitset;
 
     auto arg0 = Args.evaluate(0);
-    const auto& A = arg0. as_<alignment>();
+    const auto& A = arg0. as_<Box<alignment>>().value();
 
     int seq = Args.evaluate(1).as_int();
 
@@ -290,11 +319,164 @@ extern "C" closure builtin_function_bitmask_from_alignment(OperationArgs& Args)
     return mask_;
 }
 
+
+extern "C" closure builtin_function_compress_alignment(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& A1 = arg0.as_checked<alignment>();
+
+    int n = Args.evaluate(1).as_int();
+
+    auto [A,counts,mapping] = compress_alignment(A1,n);
+
+    object_ptr<EVector> result(new EVector);
+    result->resize(3);
+    (*result)[0] = Box<alignment>(A);
+    (*result)[1] = EVector(counts);
+    (*result)[2] = EVector(mapping);
+
+    return result;
+}
+
+extern "C" closure builtin_function_leaf_sequence_counts(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& A = arg0.as_checked<alignment>();
+
+    int n = Args.evaluate(1).as_int();
+
+    auto arg2 = Args.evaluate(2);
+    auto counts = (vector<int>)arg2.as_checked<EVector>();
+
+    EVector all_counts;
+    for(int i=0; i < n; i++)
+        all_counts.push_back(EVector(alignment_row_counts(A,i,counts)));
+
+    return all_counts;
+}
+
 extern "C" closure builtin_function_load_alignment(OperationArgs& Args)
 {
-    std::string filename = Args.evaluate(0).as_<String>();
+    auto arg0 = Args.evaluate(0);
+    auto& a = *arg0.as_checked<Alphabet>();
 
-    object_ptr<alignment> A(new alignment(DNA(),filename));
+    string filename = Args.evaluate(1).as_checked<String>();
+
+    object_ptr<Box<alignment>> A(new Box<alignment>(a,filename));
 
     return A;
 }
+
+extern "C" closure builtin_function_load_sequences(OperationArgs& Args)
+{
+    string filename = Args.evaluate(0).as_checked<String>();
+
+    auto sequences_ = sequence_format::load_from_file(filename);
+
+    EVector sequences(sequences_.size());
+    for(int i=0;i<sequences.size();i++)
+        sequences[i] = new Box<sequence>(sequences_[i]);
+
+    return sequences;
+}
+
+extern "C" closure builtin_function_select_range(OperationArgs& Args)
+{
+    string range = Args.evaluate(0).as_checked<String>();
+
+    auto arg1 = Args.evaluate(1);
+    auto& sequences = arg1.as_<EVector>();
+
+    // 1. Get maximum length
+    int L = 0;
+    for(auto& s: sequences)
+	L = std::max<int>(L, s.as_<Box<sequence>>().size());
+
+    // 2. Find columns
+    vector<int> columns = parse_multi_range(range, L);
+
+    EVector sequences2;
+    for(auto& e: sequences)
+    {
+        auto& s = e.as_<Box<sequence>>();
+        // unshare
+        auto s2 = new Box<sequence>(s);
+        // make empty
+        s2->string::operator=("");
+	for(int col : columns)
+            if (col < s.size())
+                (*s2) += s[col];
+        sequences2.push_back(s2);
+    }
+
+    return sequences2;
+}
+
+extern "C" closure builtin_function_alignment_from_sequences(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& a = *arg0.as_checked<Alphabet>();
+
+    auto arg1 = Args.evaluate(1);
+    auto& sequences_ = arg1.as_<EVector>();
+
+    vector<sequence> sequences;
+    for(auto& s: sequences_)
+        sequences.push_back(s.as_<Box<sequence>>());
+
+    object_ptr<Box<alignment>> A(new Box<alignment>(a));
+
+    A->load(sequences);
+
+    return A;
+}
+
+
+extern "C" closure builtin_function_sequences_from_alignment(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& A = arg0.as_<Box<alignment>>().value();
+    auto &a = A.get_alphabet();
+
+    EVector sequences;
+    for(int i=0;i<A.n_sequences();i++)
+    {
+	EVector seq;
+	for(int col=0;col<A.length();col++)
+	    if (auto l = A(col,i); a.is_feature(l))
+		seq.push_back(l);
+	sequences.push_back(std::move(seq));
+    }
+
+    return sequences;
+}
+
+extern "C" closure builtin_function_sequence_names(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& A = arg0.as_<Box<alignment>>().value();
+
+    EVector sequence_names;
+    for(int i=0;i<A.n_sequences();i++)
+	sequence_names.push_back(String(A.seq(i).name));
+
+    return sequence_names;
+}
+
+extern "C" closure builtin_function_reorder_alignment(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& names = arg0.as_<EVector>();
+
+    auto arg1 = Args.evaluate(1);
+    auto& A1 = arg1.as_<Box<alignment>>().value();
+
+    vector<string> sequence_names;
+    for(auto& name: names)
+        sequence_names.push_back(name.as_<String>());
+
+    object_ptr<Box<alignment>> A2(new Box<alignment>(reorder_sequences(A1, sequence_names)));
+
+    return A2;
+}
+
