@@ -1,18 +1,18 @@
 #pragma clang diagnostic ignored "-Wreturn-type-c-linkage"
 #include "computation/machine/args.H"
-#include "computation/expression/expression.H"
+#include "computation/expression/index_var.H"
+#include "computation/expression/constructor.H"
 
 extern "C" closure builtin_function_mkArray(OperationArgs& Args)
 {
     int n = Args.evaluate(0).as_int();
-    expression_ref f = Args.reference(1);
 
     const closure& C = Args.current_closure();
 
     // We can't do negative-sized arrays
     assert(n >= 0);
     // The function should be represented as a heap variable...
-    int f_reg = C.lookup_in_env(f.as_index_var());
+    int f_reg = C.reg_for_slot(1);
   
     object_ptr<expression> exp = new expression(constructor("Array",n));
     exp->sub.resize(n);
@@ -47,7 +47,7 @@ extern "C" closure builtin_function_mkArray(OperationArgs& Args)
 
 extern "C" closure builtin_function_arraySize(OperationArgs& Args)
 {
-    int N = Args.evaluate_slot_to_closure(0).exp.size() - 1;
+    int N = Args.evaluate_slot_to_closure(0).exp.size();
 
     return {N};
 }
@@ -61,6 +61,9 @@ extern "C" closure builtin_function_getIndex(OperationArgs& Args)
     // Do this second, so that evaluation of the 1st argument can't call expand_memory afterwards.
     const closure& C = Args.evaluate_slot_to_closure(0);
 
+    if (not is_constructor(C.exp.head()) or C.exp.head().as_<constructor>().f_name != "Array")
+	throw myexception()<<"Trying to index expression that is not an Array:   "<<C.exp;
+
     int N = C.exp.size();
     assert(C.Env.size() == C.exp.size());
 
