@@ -2,9 +2,8 @@
 #include <vector>
 #include <string>
 #include "computation/machine/args.H"
-#include "myexception.H"
-#include "io.H"
-#include "computation/expression/expression.H"
+#include "util/myexception.H"
+#include "util/io.H"
 
 using boost::dynamic_pointer_cast;
 using std::vector;
