@@ -6,10 +6,8 @@
 #include "computation/machine/args.H"
 #include "computation/module.H"
 #include "probability/probability.H"
-#include "bounds.H"
-#include "rng.H"
-#include "vector_from_list.H"
-#include "computation/expression/expression.H"
+#include "util/bounds.H"
+#include "util/rng.H"
 
 using std::vector;
 using std::string;
@@ -118,10 +116,10 @@ extern "C" closure builtin_function_sample_cauchy(OperationArgs& Args)
 // Third convert all the expression_ref's here to "var" and use Distribution_Functions()
 extern "C" closure builtin_function_dirichlet_density(OperationArgs& Args)
 {
-    auto n = Args.evaluate(0);
-    auto x = Args.evaluate(1);
+    auto n = (vector<double>) Args.evaluate(0).as_<EVector>();
+    auto x = (vector<double>) Args.evaluate(1).as_<EVector>();
   
-    return { ::dirichlet_pdf(x.as_<EVector>(), n.as_<EVector>()) };
+    return { ::dirichlet_pdf(x, n) };
 }
 
 extern "C" closure builtin_function_laplace_density(OperationArgs& Args)
@@ -282,13 +280,12 @@ extern "C" closure builtin_function_CRP_density(OperationArgs& Args)
     int D = Args.evaluate(2).as_int();
 
     //------------- 2. Get argument Z -----------------
-    vector<int> z = vec_to_int (get_vector_from_list(Args,3) );
+    auto z = (vector<int>) Args.evaluate(3).as_<EVector>();
 
     return { ::CRP_pdf(alpha,N,D,z) };
 }
 
 #include "probability/choose.H"
-#include "util.H"
 
 extern "C" closure builtin_function_sample_CRP(OperationArgs& Args)
 {
@@ -345,7 +342,7 @@ extern "C" closure builtin_function_sample_CRP(OperationArgs& Args)
 extern "C" closure builtin_function_sample_categorical(OperationArgs& Args)
 {
     //------------- 1. Get argument p -----------------
-    vector<double> z = vec_to_double (get_vector_from_list_(Args,0) );
+    auto z = (vector<double>) Args.evaluate(0).as_<EVector>();
 
     return { choose_scratch(z) };
 }
