@@ -1,7 +1,11 @@
 #pragma clang diagnostic ignored "-Wreturn-type-c-linkage"
 #include "computation/machine/args.H"
-#include "computation/expression/expression.H"
-#include "vector_from_list.H"
+#include "computation/expression/index_var.H"
+#include "computation/expression/constructor.H"
+#include "computation/expression/bool.H"
+#include "computation/operations.H"
+#include "util/string/convert.H"
+#include "computation/machine/graph_register.H"
 
 using boost::dynamic_pointer_cast;
 using std::string;
@@ -75,32 +79,6 @@ extern "C" closure builtin_function_doubleToInt(OperationArgs& Args)
     return {xi};
 }
 
-extern "C" closure builtin_function_vector_from_list(OperationArgs& Args)
-{
-    object_ptr<EVector> v (new EVector);
-
-    const closure* top = &Args.evaluate_slot_to_closure(0);
-    while(top->exp.size())
-    {
-	assert(has_constructor(top->exp,":"));
-	assert(top->exp.size() == 2);
-
-	int element_index = top->exp.sub()[0].as_index_var();
-	int element_reg = top->lookup_in_env( element_index );
-
-	int next_index = top->exp.sub()[1].as_index_var();
-	int next_reg = top->lookup_in_env( next_index );
-
-	// Add the element to the list.
-	v->push_back( Args.evaluate_reg_to_object(element_reg) );
-	// Move to the next element or end
-	top = &Args.evaluate_reg_to_closure(next_reg);
-    }
-    assert(has_constructor(top->exp,"[]"));
-
-    return v;
-}
-
 extern "C" closure builtin_function_add(OperationArgs& Args)
 {
     auto x = Args.evaluate(0);
@@ -267,38 +245,100 @@ extern "C" closure builtin_function_negate(OperationArgs& Args)
 	throw myexception()<<"Negate: object '"<<x.print()<<"' is not double, int, or char'";
 }
 
-extern "C" closure builtin_function_equals(OperationArgs& Args)
+extern "C" closure builtin_function_get_n_args(OperationArgs& Args)
 {
     auto x = Args.evaluate(0);
-    auto y = Args.evaluate(1);
-  
-    if (x.is_double())
-	return {x.as_double() == y.as_double()};
-    else if (x.is_int())
-	return {x.as_int() == y.as_int()};
-    else if (x.is_log_double())
-	return {x.as_log_double() == y.as_log_double()};
-    else if (x.is_char())
-	return {x.as_char() == y.as_char()};
-    else
-	throw myexception()<<"==: object '"<<x.print()<<"' is not double, int, log_double, or char'";
+    return {expression_ref(int(x.size()))};
 }
 
-extern "C" closure builtin_function_notequals(OperationArgs& Args)
+extern "C" closure builtin_function_get_arg(OperationArgs& Args)
+{
+    auto& x = Args.evaluate_slot_to_closure(0);
+
+    int i = Args.evaluate(1).as_int();
+
+    if (i < 0 or i >= x.exp.size())
+        throw myexception()<<"Prelude:get_arg: Can't access argument "<<i<<" of '"<<x.print()<<"'";
+
+    int r = x.reg_for_slot(i);
+
+    return closure{index_var(0),{r}};
+}
+
+extern "C" closure builtin_function_equals_top(OperationArgs& Args)
+{
+    constexpr int False = 1;
+    constexpr int True =  2;
+    constexpr int DontKnow = 3;
+
+    auto x = Args.evaluate(0);
+    auto y = Args.evaluate(1);
+
+    // 1. If the number of arguments isn't equal, then x /= y
+    if (x.size() != y.size()) return {expression_ref(False)};
+
+    // 2. If the heads aren't equal, then x /= y.
+    if (x.head() != y.head()) return {expression_ref(False)};
+
+    // 3. If x and y have no arguments, then x == y
+    if (x.size() == 0) return {expression_ref(True)};
+
+    // 4. Otherwise we dont know.
+    return {expression_ref(DontKnow)};
+}
+
+enum Ordering {LT=4,GT=5,EQ=6,DontKnow=7};
+
+template <typename T>
+Ordering compare(const T& x, const T& y)
+{
+    if (x < y) return Ordering::LT;
+    if (x > y) return Ordering::GT;
+    return Ordering::EQ;
+}
+
+extern "C" closure builtin_function_compare_top(OperationArgs& Args)
 {
     auto x = Args.evaluate(0);
     auto y = Args.evaluate(1);
-  
-    if (x.is_double())
-	return {x.as_double() != y.as_double()};
-    else if (x.is_int())
-	return {x.as_int() != y.as_int()};
+
+    // 1. Compare number of arguments
+    auto cmp = compare( x.size(), y.size() );
+    if (cmp != Ordering::EQ)
+        return expression_ref(int(cmp));
+
+    // 2. Compare the heads
+    if (x.is_int())
+        cmp = compare(x.as_int(), y.as_int());
+    else if (x.is_double())
+        cmp = compare(x.as_double(), y.as_double());
     else if (x.is_log_double())
-	return {x.as_log_double() != y.as_log_double()};
+        cmp = compare(x.as_log_double(), y.as_log_double());
     else if (x.is_char())
-	return {x.as_char() != y.as_char()};
+        cmp = compare(x.as_char(), y.as_char());
+    else if (x.is_a<String>())
+        cmp = compare<string>(x.as_<String>(), y.as_<String>());
+    else if (x.head().is_a<constructor>())
+    {
+        auto xc = x.head().as_<constructor>();
+        auto yc = y.head().as_<constructor>();
+
+        assert(x.size() == xc.n_args());
+        assert(y.size() == yc.n_args());
+
+        cmp = compare<string>(xc.f_name, xc.f_name);
+    }
     else
-	throw myexception()<<"/=: object '"<<x.print()<<"' is not double, int, log_double, or char'";
+        throw myexception()<<"Prelude:compare_top: `"<<x<<"` and `"<<y<<"` can't be compared!";
+
+    if (cmp != Ordering::EQ)
+        return expression_ref(int(cmp));
+
+    // 3. If x and y have no arguments, then x == y
+    if (x.size() == 0) return {expression_ref(int(Ordering::EQ))};
+
+    // 4. Otherwise we don't know
+    return {expression_ref(int(Ordering::DontKnow))};
 }
 
 extern "C" closure builtin_function_greaterthan(OperationArgs& Args)
@@ -381,29 +421,24 @@ extern "C" closure builtin_function_intToDouble(OperationArgs& Args)
     return {double(i)};
 }
 
-#include "iota.H"
-
-extern "C" closure builtin_function_iotaUnsigned(OperationArgs& Args)
+extern "C" closure builtin_function_seq(OperationArgs& Args)
 {
-    return iota_function<unsigned>(Args);
+    int x = Args.reg_for_slot(0);
+    int y = Args.reg_for_slot(1);
+
+    expression_ref E(Seq(),{index_var(1),index_var(0)});
+
+    return closure{E,{x,y}};
 }
 
 extern "C" closure builtin_function_join(OperationArgs& Args)
 {
-    Args.evaluate_slot_to_reg(0);
-    int R = Args.evaluate_slot_to_reg(1);
+    int x = Args.reg_for_slot(0);
+    int y = Args.reg_for_slot(1);
 
-    return {index_var(0),{R}};
-}
+    expression_ref E(Join(),{index_var(1),index_var(0)});
 
-extern "C" closure builtin_function_seq(OperationArgs& Args)
-{
-    Args.evaluate_slot_no_record(0);
-
-    int index = Args.reference(1).as_index_var();
-    int R = Args.current_closure().lookup_in_env( index);
-
-    return {index_var(0),{R}};
+    return closure{E,{x,y}};
 }
 
 extern "C" closure builtin_function_show(OperationArgs& Args)
@@ -419,11 +454,7 @@ extern "C" closure builtin_function_show(OperationArgs& Args)
 
 extern "C" closure builtin_function_error(OperationArgs& Args)
 {
-    vector<char> v = vec_to_char ( get_vector_from_list(Args,0) );
-
-    string message;
-    for(auto ch: v)
-	message += ch;
+    string message = Args.evaluate(0).as_<String>();
   
     throw error_exception(message);
 }
@@ -432,36 +463,11 @@ extern "C" closure builtin_function_putStrLn(OperationArgs& Args)
 {
     string message = Args.evaluate(0).as_<String>();
 
+    int state = Args.evaluate(1).as_int();
+
     std::cout<<message<<std::endl;
 
-    return constructor("()",0);
-}
-
-extern "C" closure builtin_function_reapply(OperationArgs& Args)
-{
-    int index1 = Args.reference(0).as_index_var();
-    int R1 = Args.current_closure().lookup_in_env( index1 );
-
-    int index2 = Args.reference(1).as_index_var();
-    int R2 = Args.current_closure().lookup_in_env( index2 );
-
-    expression_ref apply_E;
-    {
-	expression_ref fE = index_var(1);
-	expression_ref argE = index_var(0);
-	apply_E = {fE, argE};
-    }
-
-    // %1 %0 {R1,R2}
-    int apply_reg = Args.allocate({apply_E,{R1, R2}});
-
-    // FIXME - aren't we trying to eliminate general evaluation of regs that aren't children?  See below:
-
-    // Evaluate the newly create application reg - and depend upon it!
-    if (Args.evaluate_changeables())
-	Args.evaluate_reg_to_object(apply_reg);
-
-    return {index_var(0),{apply_reg}};
+    return EPair(state+1, constructor("()",0));
 }
 
 extern "C" closure builtin_function_read_int(OperationArgs& Args)
