@@ -4,12 +4,13 @@
 #include "math/exponential.H"
 #include "math/eigenvalue.H"
 #include "sequence/alphabet.H"
-#include "io.H"
+#include "util/io.H"
 #include <valarray>
-#include "computation/expression/expression.H"
 #include "dp/2way.H"
+#include "util/range.H"
 
 using std::vector;
+using std::pair;
 using std::istringstream;
 using std::istream;
 using std::valarray;
@@ -18,10 +19,12 @@ using std::cerr;
 using std::endl;
 using std::abs;
 
+using Alphabet = PtrBox<alphabet>;
+
 extern "C" closure builtin_function_lExp(OperationArgs& Args)
 {
     auto L = Args.evaluate(0);
-    vector<double> pi = Args.evaluate(1).as_<EVector>();
+    auto pi = (vector<double>) Args.evaluate(1).as_<EVector>();
     double t = Args.evaluate(2).as_double();
 
     auto M = new Box<Matrix>;
@@ -94,10 +97,10 @@ extern "C" closure builtin_function_get_eigensystem(OperationArgs& Args)
 extern "C" closure builtin_function_get_equilibrium_rate(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const alphabet& a = arg0.as_<alphabet>();
+    const alphabet& a = *arg0.as_<Alphabet>();
 
     auto arg1 = Args.evaluate(1);
-    const vector<unsigned>& smap = arg1.as_< Vector<unsigned> >();
+    auto& smap = arg1.as_< EVector >();
 
     auto arg2 = Args.evaluate(2);
     const Matrix& Q = arg2.as_< Box<Matrix> >();
@@ -120,7 +123,7 @@ extern "C" closure builtin_function_get_equilibrium_rate(OperationArgs& Args)
 	{
 	    double temp = 0;
 	    for(int s2=0;s2<N;s2++)
-		if (smap[s1] != smap[s2])
+		if (smap[s1].as_int() != smap[s2].as_int())
 		    temp += Q(s1,s2);
 
 	    scale += temp*pi[s1];
@@ -133,7 +136,7 @@ extern "C" closure builtin_function_get_equilibrium_rate(OperationArgs& Args)
 extern "C" closure builtin_function_rna_16a_exchange(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Doublets& D = arg0.as_<Doublets>();
+    auto& D = *arg0.poly_as_<alphabet,Doublets>();
 
     // 2 changes: between matches/wobbles (both transitions)
     double alpha_D = Args.evaluate(2).as_double();
@@ -216,7 +219,7 @@ extern "C" closure builtin_function_rna_16a_exchange(OperationArgs& Args)
 extern "C" closure builtin_function_singlet_to_doublet_rates(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Doublets& D = arg0.as_<Doublets>();
+    auto& D = *arg0.poly_as_<alphabet,Doublets>();
 
     auto arg1 = Args.evaluate(1);
     const Matrix& R1 = arg1.as_<Box<Matrix>>();
@@ -276,7 +279,7 @@ extern "C" closure builtin_function_singlet_to_doublet_rates(OperationArgs& Args
 extern "C" closure builtin_function_singlet_to_triplet_rates(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Triplets& T = arg0.as_<Triplets>();
+    auto& T = *arg0.poly_as_<alphabet,Triplets>();
   
     auto arg1 = Args.evaluate(1);
     const Matrix& R1 = arg1.as_<Box<Matrix>>();
@@ -400,6 +403,132 @@ extern "C" closure builtin_function_equ(OperationArgs& Args)
 // when the discrete approximation to the continuous distribution has
 // low resolution.
 
+
+// Lets suppose that we have a list of `n_levels` different rate matrices with:
+//    * rate matrices Qs[leven]
+//    * pi vectors pis[level]
+//    * state maps smaps[level]
+// for k=1..K.
+
+void inc_modulated_states_matrix(int& r, int& level, int& state, const EVector& Qs)
+{
+    r++;
+    state++;
+    if (state < Qs[level].as_<Box<Matrix>>().size1())
+        ;
+    else
+    {
+        level++;
+        state=0;
+    }
+}
+
+void inc_modulated_states_vec(int& r, int& level, int& state, const EVector& pis)
+{
+    r++;
+    state++;
+    if (state < pis[level].as_<EVector>().size())
+        ;
+    else
+    {
+        level++;
+        state=0;
+    }
+}
+
+// We switch between these different "levels" with KxK rate matrix `rates_between`.
+// The equilibrium frequencies for each level in `R` are given by `level_frequencies`.
+extern "C" closure builtin_function_modulated_markov_rates(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& Qs = arg0.as_<EVector>();
+    int n_levels = Qs.size();
+
+    auto arg1 = Args.evaluate(1);
+    auto& rates_between = arg1.as_<Box<Matrix>>();
+    assert(rates_between.size1() == n_levels);
+    assert(rates_between.size2() == n_levels);
+
+    int total_states = 0;
+    for(int l = 0; l < n_levels; l++)
+    {
+        auto& Q = Qs[l].as_<Box<Matrix>>();
+        int n_states_for_level = Q.size1();
+        assert(Q.size2() == n_states_for_level);
+
+        total_states += n_states_for_level;
+    }
+
+    auto R = new Box<Matrix>(total_states, total_states);
+    for(int r1=0, l1=0, s1=0; r1 < total_states; inc_modulated_states_matrix(r1,l1,s1,Qs))
+    {
+        double sum = 0;
+        for(int r2=0, l2=0, s2=0; r2 < total_states; inc_modulated_states_matrix(r2,l2,s2,Qs))
+        {
+            if (r1 == r2) continue;
+
+            double rate = 0;
+            if (l1 != l2 and s1 != s2)
+                ;
+            else if (l1 != l2)
+                rate = rates_between(l1,l2);
+            else
+            {
+                assert(l1 == l2);
+                auto& Q = Qs[l1].as_<Box<Matrix>>();
+                assert(s1 != s2);
+                rate = Q(s1,s2);
+            }
+            assert(rate >= 0);
+            (*R)(r1,r2) = rate;
+            sum += rate;
+        }
+        (*R)(r1,r1) = -sum;
+    }
+
+    return R;
+}
+
+extern "C" closure builtin_function_modulated_markov_pi(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& pis = arg0.as_<EVector>();
+    int n_levels = pis.size();
+
+    auto arg1 = Args.evaluate(1);
+    auto& level_probs = arg1.as_<EVector>();
+    assert(level_probs.size() == n_levels);
+
+    int total_states = 0;
+    for(int l = 0; l < n_levels; l++)
+    {
+        auto& pi = pis[l].as_<EVector>();
+        int n_states_for_level = pi.size();
+
+        total_states += n_states_for_level;
+    }
+
+    vector<double> pi(total_states);
+    for(int r=0, l=0, s=0; r < total_states; inc_modulated_states_vec(r,l,s,pis))
+        pi[r] = level_probs[l].as_double() * pis[l].as_<EVector>()[s].as_double();
+
+    assert(std::abs(sum(pi) - 1.0) < 1.0e-9);
+    return EVector(pi);
+}
+
+extern "C" closure builtin_function_modulated_markov_smap(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto& smaps = arg0.as_<EVector>();
+
+    EVector new_smap;
+    for(auto& smap: smaps)
+        for(auto& x: smap.as_<EVector>())
+            new_smap.push_back(x);
+
+    return new_smap;
+}
+
 /*
   object_ptr<ReversibleMarkovModelObject> Modulated_Markov_Function(const ExchangeModelObject& S,MultiModelObject M)
   {
@@ -592,7 +721,7 @@ extern "C" closure builtin_function_empirical(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
     auto S = Args.evaluate(1);
-    return Empirical_Exchange_Function(a.as_<alphabet>(), S.as_<String>());
+    return Empirical_Exchange_Function(*a.as_<Alphabet>(), S.as_<String>());
 }
 
 object_ptr<const Object> PAM_Exchange_Function(const alphabet& a)
@@ -624,7 +753,7 @@ object_ptr<const Object> PAM_Exchange_Function(const alphabet& a)
 extern "C" closure builtin_function_pam(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
-    return PAM_Exchange_Function(a.as_<alphabet>());
+    return PAM_Exchange_Function(*a.as_<Alphabet>());
 }
 
 object_ptr<const Object> JTT_Exchange_Function(const alphabet& a)
@@ -656,7 +785,7 @@ object_ptr<const Object> JTT_Exchange_Function(const alphabet& a)
 extern "C" closure builtin_function_jtt(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
-    return JTT_Exchange_Function(a.as_<alphabet>());
+    return JTT_Exchange_Function(*a.as_<Alphabet>());
 }
 
 const char* wag_string =
@@ -688,14 +817,14 @@ extern "C" closure builtin_function_wag(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
     istringstream file(wag_string);
-    return Empirical_Exchange_Function(a.as_<alphabet>(), file);
+    return Empirical_Exchange_Function(*a.as_<Alphabet>(), file);
 }
 
 extern "C" closure builtin_function_wag_frequencies(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
     istringstream file(wag_string);
-    return Empirical_Frequencies_Function(a.as_<alphabet>(), file);
+    return Empirical_Frequencies_Function(*a.as_<Alphabet>(), file);
 }
 
 
@@ -724,20 +853,20 @@ extern "C" closure builtin_function_lg(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
     istringstream file(lg_string);
-    return Empirical_Exchange_Function(a.as_<alphabet>(), file);
+    return Empirical_Exchange_Function(*a.as_<Alphabet>(), file);
 }
 
 extern "C" closure builtin_function_lg_frequencies(OperationArgs& Args)
 {
     auto a = Args.evaluate(0);
     istringstream file(lg_string);
-    return Empirical_Frequencies_Function(a.as_<alphabet>(), file);
+    return Empirical_Frequencies_Function(*a.as_<Alphabet>(), file);
 }
 
 extern "C" closure builtin_function_f3x4_frequencies(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Triplets& T = arg0.as_<Triplets>();
+    auto& T = *arg0.poly_as_<alphabet,Triplets>();
     // The way alphabet is currently implemented, triplets must be triplets of nucleotides.
 
     auto arg1 = Args.evaluate(1);
@@ -784,7 +913,7 @@ extern "C" closure builtin_function_f3x4_frequencies(OperationArgs& Args)
 extern "C" closure builtin_function_f2x4_frequencies(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Doublets& D = arg0.as_<Doublets>();
+    auto& D = *arg0.poly_as_<alphabet,Doublets>();
     // The way alphabet is currently implemented, triplets must be triplets of nucleotides.
 
     auto arg1 = Args.evaluate(1);
@@ -826,19 +955,11 @@ extern "C" closure builtin_function_gtr_sym(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
     auto& S = arg0.as_<EVector>();
-    auto arg1 = Args.evaluate(1);
-    const alphabet& a = arg1.as_<alphabet>();
+    int n = Args.evaluate(1).as_int();
 
-    int n = a.size();
     auto R = new Box<Matrix>(n,n);
     if (S.size() != n*(n-1)/2)
-	throw myexception()<<"Alphabet size is "<<n<<" but only got "<<S.size()<<" exchangeabilities";
-
-    double total = 0;
-    for(int i=0;i<S.size();i++)
-	total += S[i].as_double();
-
-    assert(total > 0);
+	throw myexception()<<"Matrix of size "<<n<<" x "<<n<<" should have "<<n*(n-1)/2<<" exchangeabilities, but got "<<S.size()<<"!";
 
     int k=0;
     for(int i=0;i<n;i++)
@@ -846,7 +967,7 @@ extern "C" closure builtin_function_gtr_sym(OperationArgs& Args)
 	(*R)(i,i) = 0;
 	for(int j=i+1;j<n;j++)
 	{
-	    double x = S[k++].as_double()/total;
+	    double x = S[k++].as_double();
 	    (*R)(i,j) = (*R)(j,i) = x;
 	}
     }
@@ -879,6 +1000,25 @@ extern "C" closure builtin_function_fixup_diagonal_rates(OperationArgs& Args)
     return m2;
 }
 
+// Currently we are assuming that one of these matrices is symmetric, so that we don't have to update the frequencies.
+extern "C" closure builtin_function_scaleMatrix(OperationArgs& Args)
+{
+    double factor = Args.evaluate(0).as_double();;
+
+    auto arg2 = Args.evaluate(1);
+    const Matrix& m = arg2.as_<Box<Matrix>>();
+
+    int n1 = m.size1();
+    int n2 = m.size2();
+
+    auto m2 = new Box<Matrix>(n1,n2);
+    for(int i=0;i<n1;i++)
+	for(int j=0;j<n2;j++)
+	    (*m2)(i,j) = factor * m(i,j);
+
+    return m2;
+}
+
 // Currently we are assuming that one of these matrices is symmetric, so that we don't have to update the frequencies.
 extern "C" closure builtin_function_elementwise_multiply(OperationArgs& Args)
 {
@@ -905,7 +1045,7 @@ extern "C" closure builtin_function_elementwise_multiply(OperationArgs& Args)
 extern "C" closure builtin_function_dNdS_matrix(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Codons& C = arg0.as_checked<Codons>();
+    const Codons& C = *arg0.poly_as_<alphabet,Codons>();
 
     double omega = Args.evaluate(1).as_double();
 
@@ -923,7 +1063,7 @@ extern "C" closure builtin_function_dNdS_matrix(OperationArgs& Args)
 extern "C" closure builtin_function_m0(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
-    const Codons& C = arg0.as_<Codons>();
+    auto& C = *arg0.poly_as_<alphabet,Codons>();
 
     auto arg1 = Args.evaluate(1);
     const Matrix& S = arg1.as_<Box<Matrix>>();
@@ -968,21 +1108,18 @@ extern "C" closure builtin_function_m0(OperationArgs& Args)
     return R;
 }
 
-extern "C" closure builtin_function_plus_gwF(OperationArgs& Args)
+extern "C" closure builtin_function_plus_gwf_matrix(OperationArgs& Args)
 {
-    const alphabet& a = Args.evaluate(0).as_<alphabet>();
+    auto pi = vector<double>( Args.evaluate(0).as_<EVector>() );
 
     double f = Args.evaluate(1).as_double();
 
-    auto pi = vector<double>( Args.evaluate(2).as_<EVector>() );
-
-    const int n = a.size();
+    int n = pi.size();
 
     auto R = new Box<Matrix>(n,n);
 
     // compute frequencies
     normalize(pi);
-    assert(a.size() == pi.size());
     
     // compute transition rates
     valarray<double> pi_f(n);
@@ -1231,10 +1368,10 @@ using boost::dynamic_bitset;
 
 namespace substitution {
     Likelihood_Cache_Branch*
-    peel_leaf_branch(const vector<int>& sequence, const vector<int>& counts, const alphabet& a, const EVector& transition_P);
+    peel_leaf_branch(const EVector& sequence, const EVector& counts, const alphabet& a, const EVector& transition_P, const EVector& smap);
 
     Likelihood_Cache_Branch*
-    peel_leaf_branch_SEV(const vector<int>& sequence, const alphabet& a, const EVector& transition_P, const dynamic_bitset<>& mask);
+    peel_leaf_branch_SEV(const EVector& sequence, const alphabet& a, const EVector& transition_P, const dynamic_bitset<>& mask);
 }
 
 extern "C" closure builtin_function_peel_leaf_branch(OperationArgs& Args)
@@ -1243,8 +1380,9 @@ extern "C" closure builtin_function_peel_leaf_branch(OperationArgs& Args)
     auto arg1 = Args.evaluate(1);
     auto arg2 = Args.evaluate(2);
     auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
 
-    return substitution::peel_leaf_branch(arg0.as_<Vector<int>>(), arg1.as_<Vector<int>>(), arg2.as_<alphabet>(), arg3.as_<EVector>());
+    return substitution::peel_leaf_branch(arg0.as_<EVector>(), arg1.as_<EVector>(), *arg2.as_<Alphabet>(), arg3.as_<EVector>(), arg4.as_<EVector>());
 }
 
 
@@ -1255,7 +1393,7 @@ extern "C" closure builtin_function_peel_leaf_branch_SEV(OperationArgs& Args)
     auto arg2 = Args.evaluate(2);
     auto arg3 = Args.evaluate(3);
 
-    return substitution::peel_leaf_branch_SEV(arg0.as_<Vector<int>>(), arg1.as_<alphabet>(), arg2.as_<EVector>(), arg3.as_<Box<dynamic_bitset<>>>());
+    return substitution::peel_leaf_branch_SEV(arg0.as_<EVector>(), *arg1.as_<Alphabet>(), arg2.as_<EVector>(), arg3.as_<Box<dynamic_bitset<>>>());
 }
 
 
@@ -1286,7 +1424,7 @@ extern "C" closure builtin_function_alignment_index2(OperationArgs& Args)
     auto arg0 = Args.evaluate(0);
     auto arg1 = Args.evaluate(1);
 
-    return substitution::alignment_index2(arg0.as_<pairwise_alignment_t>(), arg1.as_<pairwise_alignment_t>());
+    return substitution::alignment_index2(arg0.as_<Box<pairwise_alignment_t>>(), arg1.as_<Box<pairwise_alignment_t>>());
 }
 
 extern "C" closure builtin_function_alignment_index3(OperationArgs& Args)
@@ -1295,7 +1433,7 @@ extern "C" closure builtin_function_alignment_index3(OperationArgs& Args)
     auto arg1 = Args.evaluate(1);
     auto arg2 = Args.evaluate(2);
 
-    return substitution::alignment_index3(arg0.as_<pairwise_alignment_t>(), arg1.as_<pairwise_alignment_t>(),  arg2.as_<pairwise_alignment_t>());
+    return substitution::alignment_index3(arg0.as_<Box<pairwise_alignment_t>>(), arg1.as_<Box<pairwise_alignment_t>>(),  arg2.as_<Box<pairwise_alignment_t>>());
 }
 
 extern "C" closure builtin_function_peel_internal_branch(OperationArgs& Args)
@@ -1309,8 +1447,8 @@ extern "C" closure builtin_function_peel_internal_branch(OperationArgs& Args)
 
     return substitution::peel_internal_branch(&arg0.as_<Likelihood_Cache_Branch>(),
 					      &arg1.as_<Likelihood_Cache_Branch>(),
-					      arg2.as_<pairwise_alignment_t>(),
-					      arg3.as_<pairwise_alignment_t>(),
+					      arg2.as_<Box<pairwise_alignment_t>>(),
+					      arg3.as_<Box<pairwise_alignment_t>>(),
 					      arg4.as_<EVector>(),
 					      arg5.as_<Box<Matrix>>());
 }
@@ -1329,6 +1467,51 @@ extern "C" closure builtin_function_peel_internal_branch_SEV(OperationArgs& Args
 }
 
 namespace substitution {
+    Vector<pair<int,int>> sample_root_sequence(const Likelihood_Cache_Branch& cache0,
+                                               const Likelihood_Cache_Branch& cache1,
+                                               const Likelihood_Cache_Branch& cache2,
+                                               const pairwise_alignment_t& A0,
+                                               const pairwise_alignment_t& A1,
+                                               const pairwise_alignment_t& A2,
+                                               const Matrix& F);
+
+    Vector<pair<int,int>> sample_internal_node_sequence(const Vector<pair<int,int>>& parent_seq,
+                                                        const EVector& transition_Ps,
+                                                        const Likelihood_Cache_Branch& cache1,
+                                                        const Likelihood_Cache_Branch& cache2,
+                                                        const pairwise_alignment_t& A0,
+                                                        const pairwise_alignment_t& A1,
+                                                        const pairwise_alignment_t& A2,
+                                                        const Matrix& F);
+
+    Vector<pair<int,int>> sample_leaf_node_sequence(const Vector<pair<int,int>>& parent_seq,
+                                                    const EVector& transition_Ps,
+                                                    const EVector& sequence,
+                                                    const alphabet& a,
+                                                    const EVector& smap1,
+                                                    const pairwise_alignment_t& A0,
+                                                    const Matrix& F);
+
+    Vector<pair<int,int>> sample_root_sequence_SEV(const Likelihood_Cache_Branch& cache0,
+                                                   const Likelihood_Cache_Branch& cache1,
+                                                   const Likelihood_Cache_Branch& cache2,
+                                                   const Matrix& F,
+                                                   const EVector& compressed_col_for_col);
+
+    Vector<pair<int,int>> sample_internal_node_sequence_SEV(const Vector<pair<int,int>>& parent_seq,
+                                                            const EVector& transition_Ps,
+                                                            const Likelihood_Cache_Branch& cache1,
+                                                            const Likelihood_Cache_Branch& cache2,
+                                                            const EVector& compressed_col_for_col);
+
+    Vector<pair<int,int>> sample_leaf_node_sequence_SEV(const Vector<pair<int,int>>& parent_seq,
+                                                        const EVector& transition_Ps,
+                                                        const EVector& sequence,
+                                                        const Likelihood_Cache_Branch& cache1,  // just for the mask
+                                                        const alphabet& a,
+                                                        const EVector& smap1,
+                                                        const EVector& compressed_col_for_col);
+
     log_double_t calc_root_probability(const Likelihood_Cache_Branch* LCB1,
 				       const Likelihood_Cache_Branch* LCB2,
 				       const Likelihood_Cache_Branch* LCB3,
@@ -1341,7 +1524,7 @@ namespace substitution {
 					   const Likelihood_Cache_Branch* LCB2,
 					   const Likelihood_Cache_Branch* LCB3,
 					   const Matrix& F,
-					   const vector<int>& counts);
+					   const EVector& counts);
 }
 
 extern "C" closure builtin_function_calc_root_probability(OperationArgs& Args)
@@ -1357,13 +1540,123 @@ extern "C" closure builtin_function_calc_root_probability(OperationArgs& Args)
     log_double_t Pr = substitution::calc_root_probability(&arg0.as_<Likelihood_Cache_Branch>(),
 							  &arg1.as_<Likelihood_Cache_Branch>(),
 							  &arg2.as_<Likelihood_Cache_Branch>(),
-							  arg3.as_<pairwise_alignment_t>(),
-							  arg4.as_<pairwise_alignment_t>(),
-							  arg5.as_<pairwise_alignment_t>(),
+							  arg3.as_<Box<pairwise_alignment_t>>(),
+							  arg4.as_<Box<pairwise_alignment_t>>(),
+							  arg5.as_<Box<pairwise_alignment_t>>(),
 							  arg6.as_<Box<Matrix>>());
     return {Pr};
 }
 
+extern "C" closure builtin_function_sample_root_sequence(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+    auto arg5 = Args.evaluate(5);
+    auto arg6 = Args.evaluate(6);
+
+    return substitution::sample_root_sequence(arg0.as_<Likelihood_Cache_Branch>(),
+                                              arg1.as_<Likelihood_Cache_Branch>(),
+                                              arg2.as_<Likelihood_Cache_Branch>(),
+                                              arg3.as_<Box<pairwise_alignment_t>>(),
+                                              arg4.as_<Box<pairwise_alignment_t>>(),
+                                              arg5.as_<Box<pairwise_alignment_t>>(),
+                                              arg6.as_<Box<Matrix>>());
+}
+
+extern "C" closure builtin_function_sample_root_sequence_SEV(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+    auto arg5 = Args.evaluate(5);
+    auto arg6 = Args.evaluate(6);
+
+    return substitution::sample_root_sequence_SEV(arg0.as_<Likelihood_Cache_Branch>(),
+                                              arg1.as_<Likelihood_Cache_Branch>(),
+                                              arg2.as_<Likelihood_Cache_Branch>(),
+                                              arg3.as_<Box<Matrix>>(),
+                                              arg3.as_<EVector>());
+}
+
+extern "C" closure builtin_function_sample_internal_node_sequence(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+    auto arg5 = Args.evaluate(5);
+    auto arg6 = Args.evaluate(6);
+    auto arg7 = Args.evaluate(7);
+
+    return substitution::sample_internal_node_sequence(arg0.as_<Vector<pair<int,int>>>(),
+                                                       arg1.as_<EVector>(),
+                                                       arg2.as_<Likelihood_Cache_Branch>(),
+                                                       arg3.as_<Likelihood_Cache_Branch>(),
+                                                       arg4.as_<Box<pairwise_alignment_t>>(),
+                                                       arg5.as_<Box<pairwise_alignment_t>>(),
+                                                       arg6.as_<Box<pairwise_alignment_t>>(),
+                                                       arg7.as_<Box<Matrix>>());
+}
+
+extern "C" closure builtin_function_sample_internal_node_sequence_SEV(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+
+    return substitution::sample_internal_node_sequence_SEV(arg0.as_<Vector<pair<int,int>>>(),
+                                                           arg1.as_<EVector>(),
+                                                           arg2.as_<Likelihood_Cache_Branch>(),
+                                                           arg3.as_<Likelihood_Cache_Branch>(),
+                                                           arg4.as_<EVector>());
+}
+
+extern "C" closure builtin_function_sample_leaf_node_sequence(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+    auto arg5 = Args.evaluate(5);
+    auto arg6 = Args.evaluate(6);
+
+    return substitution::sample_leaf_node_sequence(arg0.as_<Vector<pair<int,int>>>(),
+                                                   arg1.as_<EVector>(),
+                                                   arg2.as_<EVector>(),
+                                                   *arg3.as_<Alphabet>(),
+                                                   arg4.as_<EVector>(),
+                                                   arg5.as_<Box<pairwise_alignment_t>>(),
+                                                   arg6.as_<Box<Matrix>>());
+}
+
+extern "C" closure builtin_function_sample_leaf_node_sequence_SEV(OperationArgs& Args)
+{
+    auto arg0 = Args.evaluate(0);
+    auto arg1 = Args.evaluate(1);
+    auto arg2 = Args.evaluate(2);
+    auto arg3 = Args.evaluate(3);
+    auto arg4 = Args.evaluate(4);
+    auto arg5 = Args.evaluate(5);
+    auto arg6 = Args.evaluate(6);
+
+    return substitution::sample_leaf_node_sequence_SEV(arg0.as_<Vector<pair<int,int>>>(),
+                                                       arg1.as_<EVector>(),
+                                                       arg2.as_<EVector>(),
+                                                       arg3.as_<Likelihood_Cache_Branch>(),
+                                                       *arg4.as_<Alphabet>(),
+                                                       arg5.as_<EVector>(),
+                                                       arg6.as_<EVector>());
+}
+
 extern "C" closure builtin_function_calc_root_probability_SEV(OperationArgs& Args)
 {
     auto arg0 = Args.evaluate(0);
@@ -1376,7 +1669,7 @@ extern "C" closure builtin_function_calc_root_probability_SEV(OperationArgs& Arg
 							      &arg1.as_<Likelihood_Cache_Branch>(),
 							      &arg2.as_<Likelihood_Cache_Branch>(),
 							      arg3.as_<Box<Matrix>>(),
-							      arg4.as_<Vector<int>>());
+							      arg4.as_<EVector>());
     return {Pr};
 }
 
@@ -1406,8 +1699,8 @@ extern "C" closure builtin_function_peel_likelihood_1(OperationArgs& Args)
     auto arg1 = Args.evaluate(1);
     auto arg2 = Args.evaluate(2);
 
-    const auto& seq  = arg0.as_<Vector<int>>();
-    const auto& a    = arg1.as_<alphabet>();
+    const auto& seq  = arg0.as_<EVector>();
+    const auto& a    = *arg1.as_<Alphabet>();
     const auto& WF   = arg2.as_<Box<Matrix>>();
 
     // Make frequency-vector AND log(frequency)-vector
@@ -1423,7 +1716,7 @@ extern "C" closure builtin_function_peel_likelihood_1(OperationArgs& Args)
     log_double_t Pr = 1;
 
     for(auto l: seq)
-	Pr *= letter_frequency(l, a, F, LF);
+	Pr *= letter_frequency(l.as_int(), a, F, LF);
 
     return {Pr};
 }
@@ -1437,10 +1730,10 @@ extern "C" closure builtin_function_peel_likelihood_2(OperationArgs& Args)
     auto arg4 = Args.evaluate(4);
     auto arg5 = Args.evaluate(5);
 
-    const auto& seq1  = arg0.as_<Vector<int>>();
-    const auto& seq2  = arg1.as_<Vector<int>>();
-    const auto& alpha = arg2.as_<alphabet>();
-    const auto& A     = arg3.as_<pairwise_alignment_t>();
+    const auto& seq1  = arg0.as_<EVector>();
+    const auto& seq2  = arg1.as_<EVector>();
+    const alphabet& alpha = *arg2.as_<Alphabet>();
+    const auto& A     = arg3.as_<Box<pairwise_alignment_t>>();
     const auto& P     = arg4.as_<EVector>();
     const auto& WF    = arg5.as_<Box<Matrix>>();
 
@@ -1465,8 +1758,8 @@ extern "C" closure builtin_function_peel_likelihood_2(OperationArgs& Args)
     {
 	if (A.is_match(x))
 	{
-	    int l1 = seq1[i++];
-	    int l2 = seq2[j++];
+	    int l1 = seq1[i++].as_int();
+	    int l2 = seq2[j++].as_int();
 
 	    if (alpha.is_letter(l1))
 	    {
@@ -1516,12 +1809,12 @@ extern "C" closure builtin_function_peel_likelihood_2(OperationArgs& Args)
 	}
 	else if (A.is_delete(x))
 	{
-	    int l = seq1[i++];
+	    int l = seq1[i++].as_int();
 	    Pr *= letter_frequency(l, alpha, F, LF);
 	}
 	else
 	{
-	    int l = seq2[j++];
+	    int l = seq2[j++].as_int();
 	    Pr *= letter_frequency(l, alpha, F, LF);
 
 	}
