@@ -1,17 +1,19 @@
 // #define DEBUG_OPTIMIZE
 
-#include "util.H"
+#include "util/string/join.H"
 #include "expression_ref.H"
 #include "lambda.H"
 #include "apply.H"
 #include "let.H"
+#include "var.H"
 #include "case.H"
 #include "trim.H"
 #include "tuple.H"
 #include "list.H" // for char_list
 #include "bool.H"
-#include "expression.H"
+#include "index_var.H"
 #include "computation/operations.H"
+#include "computation/module.H"
 #include <set>
 #include <iterator>
 #include <map>
@@ -26,6 +28,47 @@ using std::unique_ptr;
 
 using boost::dynamic_pointer_cast;
 
+string double_to_string(double d)
+{
+    string s = convertToString(d);
+    int i = s.size()-1;
+    if (s[i] == '0' and s.find('.') != string::npos)
+    {
+        while(i>=0 and s[i] == '0')
+            i--;
+        if (s[i] == '.')
+            i++;
+        s.resize(i+1);
+    }
+    return s;
+}
+
+std::string expression_ref::print() const
+{
+    switch(type_)
+    {
+    case null_type:
+        return "[NULL]";
+    case int_type:
+        return (i<0)?"("+convertToString(i)+")":convertToString(i);
+        break;
+    case double_type:
+        return (d<0)?"("+double_to_string(d)+")":double_to_string(d);
+        break;
+    case log_double_type:
+        return "LD"+convertToString(ld);
+        break;
+    case char_type:
+        return std::string("'")+c+"'";
+        break;
+    case index_var_type:
+        return std::string("%")+convertToString(i);
+        break;
+    default:
+        return ptr()->print();
+    }
+}
+
 string print_list(const expression_ref& E)
 {
     if (not has_constructor(E,":")) std::abort();
@@ -103,7 +146,7 @@ string expression::print() const
 	    vector<string> parts;
 	    for(int i=0;i<decls.size();i++)
 		parts.push_back(decls[i].first.print() + " = " + decls[i].second.print());
-	    result += join(parts,", ");
+	    result += join(parts,"; ");
 	    result += "} in " + body.print();
 	    return result;
 	}
@@ -111,7 +154,7 @@ string expression::print() const
 	if (parse_indexed_let_expression(*this, bodies, T))
 	{
 	    result = "let {";
-	    result += join(bodies,", ");
+	    result += join(bodies,"; ");
 	    result += "} in " + T.print();
 	    return result;
 	}
@@ -130,12 +173,22 @@ string expression::print() const
 	    vector<string> parts;
 	    for(int i=0;i<vars.size();i++)
 		parts.push_back( vars[i].print() + " -> " + bodies[i].print() );
-	    result += join(parts,',');
+	    result += join(parts,"; ");
 	    result += "}";
 	    return result;
 	}
     }
 
+    // We have to do this BEFORE we compute pargs, otherwise we do everything twice, which leads to exponential growth.
+    if (head.is_a<Operator>())
+    {
+        auto& O = head.as_<Operator>();
+	if (O.name() == ":" and size() == 2)
+	{
+	    return print_list(*this);
+	}
+    }
+
     // Print the (unparenthesized) sub-expressions
     vector<string> args(1+size());
     args[0] = head.print();
@@ -145,6 +198,7 @@ string expression::print() const
     vector<string> pargs = args;
     for(int i=1;i<pargs.size();i++)
     {
+        // This avoid parenthesizing do blocks, which is probably wrong.
 	if (not sub[i-1].size()) continue;
 
 	if (sub[i-1].head().is_a<Operator>())
@@ -168,15 +222,20 @@ string expression::print() const
 	string O_name = O.name();
 	if (head.is_a<Apply>())
 	{
-#ifndef DEBUG_OPTIMIZE
+            // Don't print @ f x y, just print f x y
 	    pargs.erase(pargs.begin());
-#endif
+
+            if (size() == 3 and sub[0].is_a<var>())
+            {
+                if (auto id = sub[0].as_<var>().name; is_haskell_sym(id))
+                {
+                    pargs[0] = id; // otherwise it would be (id)
+                    std::swap(pargs[0],pargs[1]);
+                }
+            }
+
 	    return O.print_expression( pargs );
 	}
-	else if (O.name() == ":" and size() == 2)
-	{
-	    return print_list(*this);
-	}
 	else if (O.precedence() > -1 and size() == 2)
 	{
 	    if (sub[0].size())
@@ -258,7 +317,7 @@ expression_ref::expression_ref(const char* s)
 {}
 
 expression_ref::expression_ref(const std::string& s)
-    :expression_ref(expression_ref{var("Foreign.Vector.listFromString"),String(s)})
+    :expression_ref(expression_ref{var("Foreign.String.unpack_cpp_string"),String(s)})
 {}
 
 expression_ref::expression_ref(const index_var& iv):i(iv.index),type_(index_var_type) {}
@@ -313,6 +372,14 @@ EVector::operator vector<int>() const
     return v2;
 }
 
+EVector::operator vector<char>() const
+{
+    vector<char> v2(size());
+    for(int i=0;i<v2.size();i++)
+	v2[i] = (*this)[i].as_char();
+    return v2;
+}
+
 EVector::EVector(const vector<double>& v1)
 {
     resize(v1.size());
@@ -326,3 +393,21 @@ EVector::EVector(const vector<int>& v1)
     for(int i=0;i<v1.size();i++)
 	(*this)[i] = v1[i];
 }
+
+EVector::EVector(const vector<char>& v1)
+{
+    resize(v1.size());
+    for(int i=0;i<v1.size();i++)
+	(*this)[i] = v1[i];
+}
+
+EPair::EPair(const pair<int,int>& p)
+{
+    first = p.first;
+    second = p.second;
+}
+
+EPair::operator std::pair<int,int>() const
+{
+    return std::pair<int,int>(first.as_int(), second.as_int());
+}
