@@ -1,10 +1,14 @@
 #include <fstream>
 #include "graph_register.H"
 #include "computation/operations.H"
-#include "computation/expression/expression.H"
 #include "computation/expression/let.H"
 #include "computation/expression/var.H"
 #include "computation/expression/case.H"
+#include "computation/expression/modifiable.H"
+#include "computation/expression/reg_var.H"
+#include "computation/expression/random_variable.H"
+#include "computation/expression/expression.H" // for launchbury_unnormalize( )
+#include "util/set.H"
 
 using std::string;
 using std::vector;
@@ -200,22 +204,51 @@ void discover_graph_vars(const reg_heap& H, int R, map<int,expression_ref>& name
 string escape(const string& s)
 {
     string s2;
-    s2.resize(s.size()*2);
+    s2.resize(s.size()*5);
     int l=0;
     for(int i=0;i<s.size();i++)
     {
 	if (s[i] == '\n')
 	{
-	    s2[l++] = '\\';
-	    s2[l++] = 'n';
-	    continue;
+	    s2[l++] = '<';
+	    s2[l++] = 'b';
+	    s2[l++] = 'r';
+	    s2[l++] = '/';
+	    s2[l++] = '>';
 	}
-
-	bool escape_next = (s[i] == '\\') or (s[i] == '\n') or (s[i] == '"') or (s[i] == '<') or (s[i] == '>') or (s[i] == '{') or (s[i] == '}');
-
-	if (escape_next)
-	    s2[l++] = '\\';
-	s2[l++] = s[i];
+	else if (s[i] == '<')
+	{
+	    s2[l++] = '&';
+	    s2[l++] = 'l';
+	    s2[l++] = 't';
+	    s2[l++] = ';';
+	}
+	else if (s[i] == '>')
+	{
+	    s2[l++] = '&';
+	    s2[l++] = 'g';
+	    s2[l++] = 't';
+	    s2[l++] = ';';
+	}
+	else if (s[i] == '&')
+	{
+	    s2[l++] = '&';
+	    s2[l++] = 'a';
+	    s2[l++] = 'm';
+	    s2[l++] = 'p';
+	    s2[l++] = ';';
+	}
+	else if (s[i] == '"')
+	{
+	    s2[l++] = '&';
+	    s2[l++] = 'q';
+	    s2[l++] = 'u';
+	    s2[l++] = 'o';
+	    s2[l++] = 't';
+	    s2[l++] = ';';
+	}
+	else
+	    s2[l++] = s[i];
     }
     s2.resize(l);
     return s2;
@@ -256,7 +289,7 @@ expression_ref untranslate_vars(const expression_ref& E, const map<int,string>&
 	{
 	    auto loc = ids.find(E.as_<reg_var>().target);
 	    if (loc != ids.end())
-		return parameter(loc->second);  // Use parameter for everything, even dummies, to avoid checks on legal variable names.
+		return var(loc->second);  // Using var( ) here used to cause a problem if the name is not a legal Haskell identifier.
 	    else
 		return E;
 	}
@@ -289,7 +322,7 @@ expression_ref compact_graph_expression(const reg_heap& C, int R, const map<stri
 map<int,string> get_register_names(const map<string, int>& ids)
 {
     map<int,string> ids2;
-    for(const auto i:ids)
+    for(const auto& i:ids)
 	ids2[i.second] = i.first;
     return ids2;
 }
@@ -297,7 +330,7 @@ map<int,string> get_register_names(const map<string, int>& ids)
 set<string> get_names(const map<string, int>& ids)
 {
     set<string> names;
-    for(const auto i:ids)
+    for(const auto& i:ids)
 	names.insert(i.first);
     return names;
 }
@@ -313,7 +346,12 @@ map<int,string> get_constants(const reg_heap& C, int t)
     // Record some regs as being constants worthy of substituting into regs that reference them.
     for(int R: regs)
     {
-	if (reg_names.count(R)) continue;
+	if (reg_names.count(R))
+	{
+	    auto rname = reg_names.at(R);
+	    rname = get_unqualified_name(rname);
+	    if (rname.size() and rname[0] != '#') continue;
+	}
 
 	if (C[R].exp.is_index_var()) continue;
 
@@ -322,6 +360,15 @@ map<int,string> get_constants(const reg_heap& C, int t)
 	if (C[R].exp.size() == 0)
 	{
 	    string name = C[R].exp.print();
+	    if (C[R].exp.is_double())
+	    {
+		if (name.find('.') != string::npos)
+		{
+		    int len = name.find_last_not_of('0');
+		    if (name[len] == '.') len++;
+		    name = name.substr(0,len+1);
+		}
+	    }
 	    if (name.size() < 20)
 		constants[R] = name;
 	}
@@ -359,10 +406,6 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 
     map<int,string> reg_names = get_register_names(ids);
 
-    const auto& params = C.get_parameters();
-    for(const auto& p: params)
-	reg_names[p.second] = p.first;
-
     map<string,string> simplify = get_simplified_names(get_names(ids));
 
     map<int,string> constants = get_constants(C, t);
@@ -383,12 +426,12 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 	expression_ref F = C[R].exp;
 
 	bool print_record = false;
-	if (F.head().type() == operation_type or F.head().type() == constructor_type)
+	if (F.head().type() == operation_type or F.head().type() == constructor_type or is_random_variable(F))
 	{
 	    if (not is_case(F) and not F.head().is_a<Apply>())
 	    {
 		print_record = true;
-		o<<"shape = record, ";
+		o<<"shape = plain, ";
 	    }
 	}
 
@@ -396,15 +439,14 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 	string label = convertToString(R);
 	if (reg_names.count(R))
 	    label += "/" + reg_names[R];
-	label += ": ";
+	label += ":";
 
 	vector<int> targets;
 	if (print_record)
 	{
-	    label = escape(label);
+	    label = "<table border='0' cellborder='1' cellspacing='0'><tr><td>"+escape(label) + "</td>";
 
-	    label += " |";
-	    label += escape(F.head().print());
+	    label += "<td>"+escape(F.head().print())+"</td>";
 	    if (F.is_expression())
 		for(const expression_ref& E: F.sub())
 		{
@@ -413,23 +455,23 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 		    targets.push_back(R2);
 	  
 		    string reg_name = "<" + convertToString(R2) + ">";
-		    if (reg_names.count(R2))
+		    if (constants.count(R2))
+			reg_name = constants[R2] + " " + reg_name;
+		    else if (reg_names.count(R2))
 		    {
 			reg_name = reg_names[R2];
 			auto loc = simplify.find(reg_name);
 			if (loc != simplify.end())
 			    reg_name = loc->second;
 		    }
-		    else if (constants.count(R2))
-			reg_name = constants[R2] + " " + reg_name;
-		    label += "| <" + convertToString(R2) + "> " + escape(reg_name) + " ";
+
+		    label += "<td port=\"r" +convertToString(R2)+"\">" + escape(reg_name) + "</td>";
 		}
+	    label += "</tr></table>";
 	}
 	else if (F.type() == index_var_type)
 	{
-	    int index = F.as_index_var();
-
-	    int R2 = C[R].lookup_in_env( index );
+	    int R2 = C[R].reg_for_index_var();
 
 	    string reg_name = "<" + convertToString(R2) + ">";
 	    if (reg_names.count(R2))
@@ -458,13 +500,13 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 	    label = escape(wrap(label,40));
 	}
 
-	o<<"label = \""<<label<<"\"";
-//	if (C.access(R).n_heads)
+	o<<"label = <"<<label<<">";
+//	if (this is a gc root) // maybe call get_roots, and then make a set<int> of all the roots?
 //	    o<<",style=\"dashed,filled\",color=orange";
 	if (C.reg_is_changeable(R))
 	    o<<",style=\"dashed,filled\",color=red";
 
-	if (C.reg_is_changeable(R) and C.reg_has_result_value(R))
+	if (C.reg_is_changeable(R) and C.has_result(R))
 	    o<<",fillcolor=\"#007700\",fontcolor=white";
 	else if (C.reg_is_changeable(R))
 	    o<<",fillcolor=\"#770000\",fontcolor=white";
@@ -493,9 +535,9 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 		if (constants.count(R2) and not C.reg_is_changeable(R2) and not used) continue;
 
 		if (not used)
-		    o<<name<<":<"<<R2<<"> -> "<<name2<<";\n";
+		    o<<name<<":r"<<R2<<" -> "<<name2<<";\n";
 		else
-		    o<<name<<":<"<<R2<<"> -> "<<name2<<" [color=\"#007777\"];\n";
+		    o<<name<<":r"<<R2<<" -> "<<name2<<" [color=\"#007777\"];\n";
 	    }
 	}
 	else
@@ -535,17 +577,14 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 		    created_call = true;
 
 	    string name2 = "n" + convertToString(R2);
-	    if (created_call)
-		o<<name<<" -> "<<name2;
-	    else
-		o<<name<<" -> "<<name2<<" ";
-	    o<<"[";
-	    o<<"color=\"#007700\"";
-	    o<<"];";
+	    o<<name<<":e -> "<<name2<<":w ";
+	    o<<"[color=\"#007700\"]";
 	    o<<"\n";
+	    if (created_call)
+		o<<"{rank = same; "<<name<<"; "<<name2<<";}"<<std::endl;
 	}
 
-	// used_inputs
+	// used_regs
 	for(int R2: C.used_regs_for_reg(R))
 	{
 	    bool is_ref_edge_also = false;
@@ -563,6 +602,24 @@ void dot_graph_for_token(const reg_heap& C, int t, std::ostream& o)
 	    o<<"];\n";
 	}
 
+	// forced_inputs
+	for(int R2: C.forced_regs_for_reg(R))
+	{
+	    bool is_ref_edge_also = false;
+	    for(int R3: C[R].Env)
+		if (R2 == R3)
+		    is_ref_edge_also = true;
+
+	    if (is_ref_edge_also) continue;
+
+	    string name2 = "n" + convertToString(R2);
+	    o<<name<<" -> "<<name2<<" ";
+	    o<<"[";
+	    o<<"color=\"#770077\"";
+	    o<<",style=dashed";
+	    o<<"];\n";
+	}
+
     }
     o<<"}"<<std::endl;
 }
