@@ -1,9 +1,9 @@
 #include <iostream>
+#include "util/range.H" // for reverse( )
 #include <unordered_map>
-#include <boost/optional.hpp>
 #include "computation/operations.H"
 #include "computation/loader.H"
-#include "computation/expression/expression.H"
+#include "computation/expression/expression.H" // is_reglike( ) and is_WHNF( )
 #include "computation/expression/let.H"
 #include "computation/expression/case.H"
 #include "computation/expression/var.H"
@@ -12,7 +12,7 @@
 #include "computation/expression/trim.H"
 #include "computation/expression/indexify.H"
 #include "computation/expression/AST_node.H"
-#include "let-float.H"
+#include "computation/expression/constructor.H"
 #include "occurrence.H"
 #include "inliner.H"
 
@@ -136,6 +136,8 @@ var get_new_name(var x, const in_scope_set& bound_vars)
     // If bound_vars doesn't contain x, then no need to change anything.
     if (it == bound_vars.end()) return x;
 
+    assert(not x.is_exported);
+
     x.index = bound_vars.size();
     while(bound_vars.count(x) and x.index > 0)
 	x.index++;
@@ -164,6 +166,8 @@ var rename_and_bind_var(const expression_ref& Evar, substitution& S, in_scope_se
 
     bind_var(bound_vars, x2, {});
 
+    if (x.is_exported) assert(x == x2);
+
     return x2;
 }
 
@@ -825,8 +829,8 @@ expression_ref simplify(const simplifier_options& options, const expression_ref&
 }
 
 
-vector<CDecls> simplify_module(const simplifier_options& options, const map<var,expression_ref>& small_decls_in,const set<var>& small_decls_in_free_vars,
-			       const vector<CDecls>& decl_groups_in)
+vector<CDecls> simplify_module_one(const simplifier_options& options, const map<var,expression_ref>& small_decls_in,const set<var>& small_decls_in_free_vars,
+                                   const vector<CDecls>& decl_groups_in)
 {
     set<var> free_vars;
 
@@ -861,3 +865,26 @@ vector<CDecls> simplify_module(const simplifier_options& options, const map<var,
 }
 
 
+vector<CDecls> simplify_module_gently(const simplifier_options& options, const map<var,expression_ref>& small_decls_in,const set<var>& small_decls_in_free_vars,
+                                      const vector<CDecls>& decl_groups_in)
+{
+    simplifier_options options_gentle = options;
+    options_gentle.case_of_case = false;
+//    options_gentle.beta_reduction = false;  This breaks the inliner.  Should probably fix!
+    options_gentle.max_iterations = 1;
+    options_gentle.inline_threshhold = -100;
+    return simplify_module(options_gentle, small_decls_in, small_decls_in_free_vars, decl_groups_in);
+}
+
+vector<CDecls> simplify_module(const simplifier_options& options, const map<var,expression_ref>& small_decls_in,const set<var>& small_decls_in_free_vars,
+                               const vector<CDecls>& decl_groups_in)
+{
+    auto decl_groups = decl_groups_in;
+
+    for(int i = 0; i < options.max_iterations; i++)
+    {
+        decl_groups = simplify_module_one(options, small_decls_in, small_decls_in_free_vars, decl_groups);
+    }
+
+    return decl_groups;
+}
