@@ -32,16 +32,22 @@
 
 #include "mcmc.H"
 #include "logger.H"
-#include "util.H"
+#include "util/range.H"
+#include "util/mapping.H"
+#include "util/string/join.H"
 
 #include "substitution/substitution.H"    // for get_model_probabilitiesby_alignment_column( )
 
-#include "monitor.H"         // for show_smodel( )
-#include "n_indels2.H"
+#include "tree-align/n_indels2.H"
 #include "substitution/parsimony.H"
 #include "alignment/alignment-util.H"
 #include "dp/2way.H"
-#include "computation/expression/expression.H"
+#include "computation/expression/bool.H"
+#include "computation/expression/constructor.H"
+
+#include "range/v3/all.hpp"
+
+namespace views = ranges::views;
 
 using std::endl;
 using std::pair;
@@ -55,11 +61,19 @@ namespace MCMC {
     using std::make_shared;
     using std::ostream;
 
-    int SortedTableFunction::n_fields() const
+
+    vector<json> json_to_table_function::operator()(const Model& M, long)
     {
-	return F->n_fields();
+	auto values = parameter_values_children( M.get_logged_parameters() );
+
+	// Check the number of fields.
+	if (values.size() != n_fields())
+	    throw myexception()<<"Number of logged fields in "<<values.size()<<" but number of field names is "<<n_fields();
+
+	return values;
     }
 
+
     vector<string> SortedTableFunction::field_names() const
     {
 	vector<string> names = F->field_names();
@@ -80,39 +94,33 @@ namespace MCMC {
 /// Parameter values indexed by indices[i] are sorted so that the parameter values indexed
 /// by indices[0] are in increasing order.
 ///
-    vector<expression_ref> make_identifiable(const vector<expression_ref>& v,const vector< vector<int> >& indices)
+    vector<json> make_identifiable(const vector<json>& v,const vector< vector<int> >& indices)
     {
 	assert(indices.size());
 	int N = indices[0].size();
 
-	vector<expression_ref> v_sub = select(v,indices[0]);
+	auto v_sub = select(v,indices[0]);
 
-	vector<int> O = iota(N);
-	std::sort(O.begin(),O.end(), 
-		  [&v_sub](int i, int j) 
-		  {
-		      double di = v_sub[i].as_double();
-		      double dj = v_sub[j].as_double();
-		      return di < dj;
-		  });
+	vector<int> O = views::ints(0,N) | ranges::to<vector>;
+	ranges::sort(O, {}, [&](int i) { return (double)v_sub[i];});
 
-	vector<int> O_all = iota<int>(v.size());
-	for(int i=0;i<indices.size();i++) 
+	vector<int> O_all = views::ints(0, (int)v.size()) | ranges::to<vector>;
+	for(auto& I: indices)
 	{
-	    assert(indices[i].size() == N);
+	    assert(I.size() == N);
 	    for(int j=0;j<N;j++) {
-		// indices[i][j] -> indices[i][O[j]]
-		O_all[indices[i][j]] = indices[i][O[j]];
+		// I[j] -> I[O[j]]
+		O_all[I[j]] = I[O[j]];
 	    }
 	}
-	vector<expression_ref> v2 = apply_mapping(v,invert(O_all));
+	auto v2 = apply_mapping(v,invert(O_all));
 
 	return v2;
     }
 
-    vector<expression_ref> SortedTableFunction::operator()(const Model& M, long t)
+    vector<json> SortedTableFunction::operator()(const Model& M, long t)
     {
-	vector<expression_ref> v = (*F)(M,t);
+	auto v = (*F)(M,t);
 
 	for(int i=0;i<indices.size();i++)
 	    v = make_identifiable(v, indices[i]);
@@ -120,7 +128,7 @@ namespace MCMC {
 	return v;
     }
 
-    SortedTableFunction::SortedTableFunction(const TableFunction<expression_ref>& f, const std::vector< std::vector< std::vector< int> > >& i_)
+    SortedTableFunction::SortedTableFunction(const TableFunction<json>& f, const std::vector< std::vector< std::vector< int> > >& i_)
 	:F(f), indices(i_), sorted_index(f.n_fields(),-1)
     { 
 	for(int i=0;i<indices.size();i++)
@@ -153,28 +161,6 @@ namespace MCMC {
 	:function(f)
     { }
 
-    expression_ref get_computation(const Model& M, int index)
-    {
-	expression_ref result = M.evaluate(index);
-
-	if (result.is_double())
-	    return result;
-	else if (result.is_int())
-	    return result;
-	else if (result.head().is_a<constructor>())
-	{
-	    auto& c = result.head().as_<constructor>();
-	    if (is_bool_true(c))
-		return 1;
-	    else if (is_bool_false(c))
-		return 0;
-	    else
-		return -1;
-	}
-	else
-	    return -1;
-    }
-
     int alignment_length(const data_partition& P)
     {
 	auto branches = P.t().all_branches_from_node(0);
@@ -221,24 +207,32 @@ namespace MCMC {
 	return convertToString(alignment_length(P));
     }
 
-    string Get_Total_Num_Substitutions_Function::operator()(const Model& M, long)
+    int n_substs(const Parameters& P)
     {
-	const Parameters& P = dynamic_cast<const Parameters&>(M);
-
 	int total = 0;
 	for(int p=0;p<P.n_data_partitions();p++)
 	    total += n_mutations(P[p]);
-	return convertToString(total);
+        return total;
     }
 
-    string Get_Total_Num_Indels_Function::operator()(const Model& M, long)
+    string Get_Total_Num_Substitutions_Function::operator()(const Model& M, long)
     {
 	const Parameters& P = dynamic_cast<const Parameters&>(M);
+        return convertToString(n_substs(P));
+    }
 
+    int n_indels(const Parameters& P)
+    {
 	int total = 0;
 	for(int p=0;p<P.n_data_partitions();p++)
 	    total += n_indels(P[p]);
-	return convertToString(total);
+        return total;
+    }
+        
+    string Get_Total_Num_Indels_Function::operator()(const Model& M, long)
+    {
+	const Parameters& P = dynamic_cast<const Parameters&>(M);
+	return convertToString(n_indels(P));
     }
 
     string Get_Total_Total_Length_Indels_Function::operator()(const Model& M, long)
@@ -261,20 +255,20 @@ namespace MCMC {
 
     string Get_Rao_Blackwellized_Parameter_Function::operator()(const Model& M, long)
     {
-	if (parameter == -1) std::abort();
+	if (node == -1) std::abort();
 
 	owned_ptr<Model> M2 = M;
 	vector<log_double_t> Prs;
 	log_double_t total = 0;
 
-	auto cur_value = M.get_parameter_value(parameter);
+	auto cur_value = M.get_modifiable_value(node);
 
         // Record probabilities
-	for(const auto& v: values)
+	for(const auto& value: values)
 	{
-	    if (v.type() != cur_value.type())
-		throw myexception()<<"Rao-Blackwellization: Trying to set parameter with value '"<<cur_value<<"' to '"<<v<<"'";
-	    M2->set_parameter_value(parameter, v);
+	    if (value.type() != cur_value.type())
+		throw myexception()<<"Rao-Blackwellization: Trying to set parameter with value '"<<cur_value<<"' to '"<<value<<"'";
+	    M2->set_modifiable_value(node, value);
 	    log_double_t Pr = M2->probability();
 	    total += Pr;
 	    Prs.push_back(Pr);
@@ -310,8 +304,8 @@ namespace MCMC {
 	return convertToString( result );
     }
 
-    Get_Rao_Blackwellized_Parameter_Function::Get_Rao_Blackwellized_Parameter_Function(int p, const vector<expression_ref>& v)
-	:parameter(p), values(v)
+    Get_Rao_Blackwellized_Parameter_Function::Get_Rao_Blackwellized_Parameter_Function(int n, const vector<expression_ref>& v)
+	:node(n), values(v)
     { }
 
     string Get_Tree_Length_Function::operator()(const Model& M, long)
@@ -355,15 +349,6 @@ namespace MCMC {
 	return output.str();
     }
 
-    string Show_SModels_Function::operator()(const Model& M, long)
-    {
-	const Parameters& P = dynamic_cast<const Parameters&>(M);
-	std::ostringstream output;
-	show_smodels(output, P);
-	output<<"\n";
-	return output.str();
-    }
-
     string Subsample_Function::operator()(const Model& M, long t)
     {
 	if (t%subsample == 0) 
@@ -403,18 +388,20 @@ namespace MCMC {
 	    return output.str();
 	}
 
-	const vector<unsigned> smap = P[p].state_letters();
+	auto smap = P[p].state_letters();
 
-	vector<vector<pair<int,int> > > states = substitution::sample_ancestral_states(P[p]);
+        auto states = P[p].ancestral_sequences();
     
 	for(int i=0;i<A.n_sequences();i++)
 	{
+            auto& node_states = states[i].as_<Vector<pair<int,int>>>();
+
 	    vector<int> columns = A.get_columns_for_characters(i);
-	    assert(columns.size() == states[i].size());
+	    assert(columns.size() == node_states.size());
 
 	    auto& seq = A.seq(i);
 	    // The length of the observed sequence should be equal to the length of the sampled sequence.
-	    assert(seq.size() == 0 or seq.size() == states[i].size());
+	    assert(seq.size() == 0 or seq.size() == node_states.size());
 
 	    // Technically we could have 0-length observed sequences -- but they would have no letters.
 	    bool observed = seq.size() > 0;
@@ -423,8 +410,8 @@ namespace MCMC {
 
 	    for(int j=0;j<columns.size();j++)
 	    {
-		int state = states[i][j].second;
-		int letter = smap[state];
+		int state = node_states[j].second;
+		int letter = smap[state].as_int();;
 
 		int c = columns[j];
 		assert(A.character(c,i));
@@ -444,16 +431,6 @@ namespace MCMC {
 	return output.str();
     }
 
-    void FunctionLogger::operator()(const Model& M, long t)
-    {
-	(*log_file)<<function(M,t);
-	log_file->flush();
-    }
-
-    FunctionLogger::FunctionLogger(const std::string& filename, const logger_function<string>& L)
-	:log_file(new checked_ofstream(filename,false)),function(L)
-    { }
-
     string ConcatFunction::operator()(const Model& M, long t)
     {
 	string output;
