@@ -23,8 +23,7 @@
 #include "util/assert.hh"
 #include "sample.H"
 #include "probability/choose.H"
-#include "util.H"
-#include "rng.H"
+#include "util/rng.H"
 #include "dp/5way.H"
 #include "dp/alignment-sums.H"
 #include "alignment/alignment-util.H"
@@ -148,16 +147,16 @@ void two_way_topology_slice_sample(owned_ptr<Model>& P, MoveStats& Stats, int b)
     A5::hmm_order order = A5::get_nodes_random(PP.t(), b);
     const auto& nodes = order.nodes;
 
-    // P.likelihood();  Why does this not make a difference in speed?
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
+
+    int b1 = PP.t().find_branch(nodes[4],nodes[1]);
+    int b2 = PP.t().find_branch(nodes[5],nodes[2]);
 
     vector<Parameters> p(2,PP);
 
-    int b1 = p[1].t().find_branch(nodes[4],nodes[1]);
-    int b2 = p[1].t().find_branch(nodes[5],nodes[2]);
-
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2);
-    p[1].select_root(b);
   
     //  if (not extends(p[1].t(), PP.PC->TC))
     //    return;
@@ -220,16 +219,16 @@ void two_way_topology_sample(owned_ptr<Model>& P, MoveStats& Stats, int b)
     A5::hmm_order order = A5::get_nodes_random(PP.t(), b);
     const auto& nodes = order.nodes;
 
-    // PP.likelihood();  Why does this not make a difference in speed?
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
+
+    int b1 = PP.t().find_branch(nodes[4],nodes[1]);
+    int b2 = PP.t().find_branch(nodes[5],nodes[2]);
 
     vector<Parameters> p(2,PP);
 
-    int b1 = p[1].t().find_branch(nodes[4],nodes[1]);
-    int b2 = p[1].t().find_branch(nodes[5],nodes[2]);
-
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2);
-    p[1].select_root(b);
   
     //  if (not extends(p[1].t(), PP.PC->TC))
     //    return;
@@ -314,16 +313,16 @@ void two_way_NNI_SPR_sample(owned_ptr<Model>& P, MoveStats& Stats, int b)
     A5::hmm_order order = A5::get_nodes_random(PP.t(), b);
     const auto& nodes = order.nodes;
 
-    // PP.likelihood();  Why does this not make a difference in speed?
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
+
+    int b1 = PP.t().find_branch(nodes[4],nodes[1]);
+    int b2 = PP.t().find_branch(nodes[5],nodes[2]);
 
     vector<Parameters> p(2,PP);
 
-    int b1 = p[1].t().find_branch(nodes[4],nodes[1]);
-    int b2 = p[1].t().find_branch(nodes[5],nodes[2]);
-
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2);
-    p[1].select_root(b);
   
     //  if (not extends(p[1].t(), PP.PC->TC))
     //    return;
@@ -380,17 +379,17 @@ void two_way_NNI_and_branches_sample(owned_ptr<Model>& P, MoveStats& Stats, int
     A5::hmm_order order = A5::get_nodes_random(PP.t(), b);
     const auto& nodes = order.nodes;
 
-    // PP.likelihood();  Why does this not make a difference in speed?
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
+
+    int b1 = PP.t().find_branch(nodes[4],nodes[1]);
+    int b2 = PP.t().find_branch(nodes[5],nodes[2]);
 
     vector<Parameters> p(2,PP);
 
     //---------------- Do the NNI operation -------------------//
-    int b1 = p[1].t().find_branch(nodes[4],nodes[1]);
-    int b2 = p[1].t().find_branch(nodes[5],nodes[2]);
-
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2);
-    p[1].select_root(b);
   
     //  if (not extends(p[1].t(), PP.PC->TC))
     //    return;
@@ -450,29 +449,6 @@ void two_way_NNI_sample(owned_ptr<Model>& P, MoveStats& Stats, int b)
 	two_way_NNI_and_branches_sample(P,Stats,b);
 }
 
-/// This has to be Gibbs, and use the same substitution::Model in each case...
-
-int three_way_topology_sample(vector<Parameters>& p, const vector<log_double_t>& rho, int b) 
-{
-    assert(p[0].variable_alignment() == p[1].variable_alignment());
-    assert(p[1].variable_alignment() == p[2].variable_alignment());
-
-    vector< A5::hmm_order > orders(3);
-    orders[0] = A5::get_nodes_random(p[0].t(), b);
-    orders[1] = A5::get_nodes_random(p[1].t(), b);
-    orders[2] = A5::get_nodes_random(p[2].t(), b);
-
-    try {
-	return sample_two_nodes_multi(p,orders,rho,true,false);
-    }
-    catch (choose_exception<log_double_t>& c)
-    {
-	c.prepend(__PRETTY_FUNCTION__);
-	throw c;
-    }
-}
-
-
 void three_way_topology_sample_slice(owned_ptr<Model>& P, MoveStats& Stats, int b) 
 {
     Parameters& PP = *P.as<Parameters>();
@@ -484,27 +460,20 @@ void three_way_topology_sample_slice(owned_ptr<Model>& P, MoveStats& Stats, int
     const auto& nodes = order.nodes;
 
     //------ Generate Topologies and alter caches ------///
-    // PP.likelihood();  Why does this not make a difference in speed?
-  
-    vector<Parameters> p(3,PP);
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
 
     int b1 = PP.t().find_branch(nodes[4],nodes[1]);
     int b2 = PP.t().find_branch(nodes[5],nodes[2]);
     int b3 = PP.t().find_branch(nodes[5],nodes[3]);
 
+    vector<Parameters> p(3,PP);
+
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2);
-    p[1].select_root(b);
-
-    //  if (not extends(p[1].t(), PP.PC->TC))
-    //    return;
 
     // Internal node states may be inconsistent after this: p[2].alignment_prior() undefined!
     p[2].NNI(b1, b3);
-    p[2].select_root(b);
-  
-    //  if (not extends(p[2].t(), PP.PC->TC))
-    //    return;
 
     const vector<log_double_t> rho(3,1);
 
@@ -575,49 +544,56 @@ void three_way_topology_sample(owned_ptr<Model>& P, MoveStats& Stats, int b)
 
     A5::hmm_order order = A5::get_nodes_random(PP.t(), b);
     const auto& nodes = order.nodes;
-
-    //------ Generate Topologies and alter caches ------///
-    // PP.likelihood();  Why does this not make a difference in speed?
-
-    vector<Parameters> p(3,PP);
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
 
     int b1 = PP.t().find_branch(nodes[4],nodes[1]);
     int b2 = PP.t().find_branch(nodes[5],nodes[2]);
     int b3 = PP.t().find_branch(nodes[5],nodes[3]);
 
+    //------ Generate Topologies and alter caches ------///
+    vector<Parameters> p(3,PP);
+
+    double L0 = PP.t().branch_length(b);
+
+    vector< A5::hmm_order > orders(3);
+    orders[0] = A5::get_nodes_random(p[0].t(), b);
+
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2);
-    p[1].select_root(b);
-
-    //  if (not extends(p[1].t(), PP.PC->TC))
-    //    return;
+    orders[1] = A5::get_nodes_random(p[1].t(), b);
 
     // Internal node states may be inconsistent after this: p[2].alignment_prior() undefined!
     p[2].NNI(b1, b3);
-    p[2].select_root(b);
+    orders[2] = A5::get_nodes_random(p[2].t(), b);
   
-    //  if (not extends(p[2].t(), PP.PC->TC))
-    //    return;
-
     const vector<log_double_t> rho(3,1);
 
     //------ Resample alignments and select topology -----//
-    int C = three_way_topology_sample(p,rho,b);
 
-    if (C != -1) {
-	PP = p[C];
-    }    
+    try {
+	int C = sample_two_nodes_multi(p,orders,rho,true,false);
 
-    MCMC::Result result(2);
+        if (C != -1) {
+            PP = p[C];
+        }
 
-    result.totals[0] = (C>0)?1:0;
-    // This gives us the average length of branches prior to successful swaps
-    if (C>0)
-	result.totals[1] = p[0].t().branch_length(b);
-    else
-	result.counts[1] = 0;
+        MCMC::Result result(2);
 
-    NNI_inc(Stats,"NNI (3-way)", result, p[0].t().branch_length(b));
+        result.totals[0] = (C>0)?1:0;
+        // This gives us the average length of branches prior to successful swaps
+        if (C>0)
+            result.totals[1] = L0;
+        else
+            result.counts[1] = 0;
+
+        NNI_inc(Stats,"NNI (3-way)", result, L0);
+    }
+    catch (choose_exception<log_double_t>& c)
+    {
+	c.prepend(__PRETTY_FUNCTION__);
+	throw c;
+    }
 }
 
 void three_way_topology_and_alignment_sample(owned_ptr<Model>& P, MoveStats& Stats, int b) 
@@ -634,23 +610,23 @@ void three_way_topology_and_alignment_sample(owned_ptr<Model>& P, MoveStats& Sta
     //--------- Generate the Different Topologies -------//
     // We ALWAYS resample the connection between two_way_nodes [0] and [4].
 
+    double L0 = PP.t().branch_length(b);
+
+    PP.select_root(b);
+    PP.cache_likelihood_branches();
+
+    int b1 = PP.t().find_branch(two_way_nodes[4],two_way_nodes[1]);
+    int b2 = PP.t().find_branch(two_way_nodes[5],two_way_nodes[2]);
+    int b3 = PP.t().find_branch(two_way_nodes[5],two_way_nodes[3]);
+
     vector<Parameters> p(3,PP);
-    int b1 = p[0].t().find_branch(two_way_nodes[4],two_way_nodes[1]);
-    int b2 = p[0].t().find_branch(two_way_nodes[5],two_way_nodes[2]);
-    int b3 = p[0].t().find_branch(two_way_nodes[5],two_way_nodes[3]);
 
     // Internal node states may be inconsistent after this: p[1].alignment_prior() undefined!
     p[1].NNI(b1, b2, true);
 
-    //  if (not extends(p[1].t(), PP.PC->TC))
-    //    return;
-
     // Internal node states may be inconsistent after this: p[2].alignment_prior() undefined!
     p[2].NNI(b1, b3, true);
 
-    //  if (not extends(p[2].t(), PP.PC->TC))
-    //    return;
-
     vector< vector< int> > nodes;
     for(int i=0;i<p.size();i++)
 	nodes.push_back(A3::get_nodes_branch_random(p[i].t(), two_way_nodes[4], two_way_nodes[0]) );
@@ -676,9 +652,9 @@ void three_way_topology_and_alignment_sample(owned_ptr<Model>& P, MoveStats& Sta
     result.totals[0] = (C>0)?1:0;
     // This gives us the average length of branches prior to successful swaps
     if (C>0)
-	result.totals[1] = p[0].t().branch_length(b);
+	result.totals[1] = L0;
     else
 	result.counts[1] = 0;
   
-    NNI_inc(Stats,"NNI (3-way) + A", result, p[0].t().branch_length(b));
+    NNI_inc(Stats,"NNI (3-way) + A", result, L0);
 }
