@@ -23,18 +23,18 @@
 /// \brief Contains functions for subtree-prune-and-regraft (SPR) transition kernels.
 ///
 
+#include "dp/2way.H"
 #include <cmath>
 #include "util/assert.hh"
 #include <iostream>
-#include <boost/optional.hpp>
 #include "sample.H"
-#include "rng.H"
+#include "util/rng.H"
 #include "probability/choose.H"
 #include "probability/probability.H"
 
 #include "dp/3way.H"
 #include "tree/tree-util.H"
-#include "util-random.H"
+#include "util/permute.H"
 #include "dp/alignment-sums.H"
 #include "alignment/alignment-constraint.H"
 #include "substitution/substitution.H"
@@ -47,7 +47,8 @@ using std::string;
 using std::map;
 using std::pair;
 using std::tuple;
-using boost::optional;
+using std::optional;
+using std::shared_ptr;
 
 using std::cout;
 using std::cerr;
@@ -144,7 +145,7 @@ int topology_sample_SPR_slice_slide_node(vector<Parameters>& p,int b)
 
     vector<double> X0(2);
     X0[0] = logp1.current_value();
-    X0[1] = uniform()*(logp2.upper_bound);
+    X0[1] = uniform()*(*logp2.upper_bound);
 
     std::pair<int,double> choice = slice_sample_multi(X0,logp,w,-1);
 
@@ -478,6 +479,7 @@ MCMC::Result sample_SPR(Parameters& P, int b1, int b2, bool slice = false)
     int n2 = P.t().source(b1);
 
     P.set_root(n1);
+    P.cache_likelihood_branches();
     vector<Parameters> p(2,P);
 
     // 2. ----- Generate the node order for 3-way alignment paths P.t().target(b1)) -------- //
@@ -970,6 +972,36 @@ move_pruned_subtree(Parameters& P,
 vector<optional<vector<HMM::bitmask_t>>> A23_constraints(const Parameters& P, const vector<int>& nodes, bool original);
 log_double_t pr_sum_out_A_tri(Parameters P, const vector<optional<vector<HMM::bitmask_t>>>& a23, const vector<int>& nodes);
 
+void set_attachment_probability(spr_attachment_probabilities& Pr, const spr_attachment_points& locations, const tree_edge& subtree_edge, const tree_edge& target_edge, Parameters p2, const map<tree_edge,vector<int>>& nodes, const tuple<int,int,int,vector<optional<vector<HMM::bitmask_t>>>>& alignment3way, bool sum_out_A)
+{
+    // ---------------- Compute the attachment probability -----------------------
+    double L = p2.t().branch_length(p2.t().find_branch(target_edge));
+    auto& nodes_ = nodes.at(target_edge);
+
+    // 0. Save constraints before we regraft
+    vector<optional<vector<HMM::bitmask_t>>> a23_constraint;
+    if (sum_out_A)
+        a23_constraint = A23_constraints(p2, nodes_, false);
+
+    // 1. Regraft subtree and set pairwise alignments on three branches
+    regraft_subtree_and_set_3way_alignments(p2, subtree_edge, target_edge, alignment3way, not sum_out_A);
+
+    // 2. Set branch lengths
+    int n0 = subtree_edge.node2;
+    set_lengths_at_location(p2, n0, L, target_edge, locations);
+
+    // 3. Compute likelihood and probability
+    if (sum_out_A)
+        Pr[target_edge] = pr_sum_out_A_tri(p2, a23_constraint, nodes_);
+    else
+        Pr[target_edge] = p2.heated_likelihood() * p2.prior();
+
+#ifdef DEBUG_SPR_ALL
+    Pr.LLL[target_edge] = p2.heated_likelihood();
+#endif
+
+}
+
 /// Compute the probability of pruning b1^t and regraftion at \a locations
 ///
 /// After this routine, likelihood caches and subalignment indices for branches in the
@@ -987,7 +1019,7 @@ SPR_search_attachment_points(Parameters P, const tree_edge& subtree_edge, const
     // This node will move around, but we will always peel up to this node to calculate the likelihood.
     int root_node = subtree_edge.node2;
     // Because the attachment node keeps its name, this will stay in effect throughout the likelihood calculations.
-    P.set_root(root_node);
+
     P.variable_alignment(false);
 
     spr_info I(P.t(), subtree_edge, range);
@@ -995,18 +1027,24 @@ SPR_search_attachment_points(Parameters P, const tree_edge& subtree_edge, const
     if (I.n_attachment_branches() == 1) return spr_attachment_probabilities();
 
     /*----------------------- Initialize likelihood for each attachment point ----------------------- */
-    vector<Parameters> Ps;
-    Ps.reserve(I.attachment_branch_pairs.size());
-    Ps.push_back(P);
-
     spr_attachment_probabilities Pr;
     if (sum_out_A)
     {
 	auto& nodes_ = nodes.at(I.initial_edge);
+        P.set_root(root_node);
 	Pr[I.initial_edge] = pr_sum_out_A_tri(P, A23_constraints(P, nodes_, true), nodes_);
     }
     else
+    {
 	Pr[I.initial_edge] = P.heated_likelihood() * P.prior();
+        P.set_root(root_node);
+        for(int j=0;j<P.n_data_partitions();j++)
+        {
+            int b0 = P.t().find_branch(subtree_edge);
+            P[j].cache(b0);
+            P[j].transition_P(b0);
+        }
+    }
 #ifdef DEBUG_SPR_ALL
     Pr.LLL[I.initial_edge] = P.heated_likelihood();
 #endif
@@ -1015,54 +1053,61 @@ SPR_search_attachment_points(Parameters P, const tree_edge& subtree_edge, const
     alignments3way.reserve(I.attachment_branch_pairs.size());
 
     // 1. Prune subtree and store homology bitpath
-    alignments3way.push_back(prune_subtree_and_get_3way_alignments(Ps[0], subtree_edge, I.initial_edge, nodes.at(I.initial_edge), not sum_out_A));
+    alignments3way.push_back(prune_subtree_and_get_3way_alignments(P, subtree_edge, I.initial_edge, nodes.at(I.initial_edge), not sum_out_A));
+
+    int x0 = P.t().find_branch(I.initial_edge);
+    int x1 = P.t().reverse(x0);
+
+    if (not P.t().is_leaf_node(I.initial_edge.node2))
+        for(int j=0;j<P.n_data_partitions();j++)
+        {
+            P[j].transition_P(x0);
+            P[j].cache(x0);
+        }
+
+    if (not P.t().is_leaf_node(I.initial_edge.node1))
+        for(int j=0;j<P.n_data_partitions();j++)
+        {
+            P[j].transition_P(x1);
+            P[j].cache(x1);
+        }
+
+    vector<Parameters> Ps;
+    Ps.reserve(I.attachment_branch_pairs.size());
+    Ps.push_back(P);
 
     // 2. Move to each attachment branch and compute homology bitpath, but don't attch
     for(int i=1;i<I.attachment_branch_pairs.size();i++)
     {
 	// Define target branch b2 - pointing away from subtree_edge
 	const auto& BB = I.attachment_branch_pairs[i];
-	const tree_edge& next_target_edge = BB.edge;
+	const tree_edge& target_edge = BB.edge;
+        const tree_edge& sibling_edge = BB.sibling;
 
 	int prev_i = BB.prev_i;
 	const tree_edge& prev_target_edge = I.attachment_branch_pairs[prev_i].edge;
 
-	if (prev_i != 0) assert(prev_target_edge.node2 == next_target_edge.node1);
+	if (prev_i != 0) assert(prev_target_edge.node2 == target_edge.node1);
 	Ps.push_back(Ps[prev_i]);
 	assert(Ps.size() == i+1);
-	auto& p = Ps.back();
-	alignments3way.push_back( move_pruned_subtree(p, alignments3way[prev_i], subtree_edge, prev_target_edge, next_target_edge, BB.sibling, not sum_out_A) );
+	auto& p0 = Ps.back();
+	auto alignment_3way = move_pruned_subtree(p0, alignments3way[prev_i], subtree_edge, prev_target_edge, target_edge, sibling_edge, not sum_out_A);
+
+        int b = p0.t().find_branch(BB.edge);
+        if (not p0.t().is_leaf_node(BB.edge.node2))
+            for(int j=0;j<p0.n_data_partitions();j++)
+            {
+                p0[j].transition_P(b);
+                p0[j].cache(b);
+            }
+
+        set_attachment_probability(Pr, locations, subtree_edge, target_edge, Ps[i], nodes, alignment_3way, sum_out_A);
+
+	alignments3way.push_back( std::move(alignment_3way) );
     }
 
-    // 3. Attach at each attachment branch at compute probabilities
     for(int i=(int)I.attachment_branch_pairs.size()-1;i>0;i--)
     {
-	const tree_edge& target_edge = I.attachment_branch_pairs[i].edge;
-	auto& p = Ps[i];
-	double L = p.t().branch_length(p.t().find_branch(target_edge));
-	auto& nodes_ = nodes.at(target_edge);
-
-	// 0. Save constraints before we regraft
-	vector<optional<vector<HMM::bitmask_t>>> a23_constraint;
-	if (sum_out_A)
-	    a23_constraint = A23_constraints(p, nodes_, false);
-
-	// 1. Regraft subtree and set pairwise alignments on three branches
-	regraft_subtree_and_set_3way_alignments(p, subtree_edge, target_edge, alignments3way[i], not sum_out_A);
-
-        // 2. Set branch lengths
-	int n0 = subtree_edge.node2;
-	set_lengths_at_location(p, n0, L, target_edge, locations);
-
-	// 3. Compute likelihood and probability
-	if (sum_out_A)
-	    Pr[target_edge] = pr_sum_out_A_tri(p, a23_constraint, nodes_);
-	else
-	    Pr[target_edge] = p.heated_likelihood() * p.prior();
-
-#ifdef DEBUG_SPR_ALL
-	Pr.LLL[target_edge] = p.heated_likelihood();
-#endif
 	Ps.pop_back();
     }
 
@@ -1101,11 +1146,11 @@ bool SPR_accept_or_reject_proposed_tree(Parameters& P, vector<Parameters>& p,
     nodes[1] = nodes_for_branch.at(I.attachment_branch_pairs[C].edge);              // in this formulation.
     bool do_cube = (uniform() < p[0].load_value("cube_fraction",0.0));
 
-    boost::shared_ptr<sample_A3_multi_calculation> tri;
+    shared_ptr<sample_A3_multi_calculation> tri;
     if (do_cube)
-	tri = boost::shared_ptr<sample_A3_multi_calculation>(new sample_cube_multi_calculation(p, nodes, true, true));
+	tri = shared_ptr<sample_A3_multi_calculation>(new sample_cube_multi_calculation(p, nodes, true, true));
     else
-	tri = boost::shared_ptr<sample_A3_multi_calculation>(new sample_tri_multi_calculation(p, nodes, true, true));
+	tri = shared_ptr<sample_A3_multi_calculation>(new sample_tri_multi_calculation(p, nodes, true, true));
     tri->run_dp();
 
     //--------- Compute PrL2: reverse proposal probabilities ---------//
@@ -1115,6 +1160,7 @@ bool SPR_accept_or_reject_proposed_tree(Parameters& P, vector<Parameters>& p,
     if (P.variable_alignment() and not sum_out_A)
 #endif
     {
+        p[1].cache_likelihood_branches();
 	spr_attachment_probabilities PrB2 = SPR_search_attachment_points(p[1], E_parent, I.range, locations, nodes_for_branch, sum_out_A);
 	vector<log_double_t> Pr2 = I.convert_to_vector(PrB2);
     
@@ -1177,11 +1223,11 @@ void spr_to_index(Parameters& P, spr_info& I, int C, const vector<int>& nodes0)
     for(int j=1;j<indices.size();j++)
     {
 	const auto& BB = I.attachment_branch_pairs[indices[j]];
-	const tree_edge& next_target_edge = BB.edge;
+	const tree_edge& target_edge = BB.edge;
 	const tree_edge& sibling_edge = BB.sibling;
 	const tree_edge& prev_target_edge = I.attachment_branch_pairs[BB.prev_i].edge;
 
-	alignments3way.push_back( move_pruned_subtree(P, alignments3way[j-1], subtree_edge, prev_target_edge, next_target_edge, sibling_edge, false) );
+	alignments3way.push_back( move_pruned_subtree(P, alignments3way[j-1], subtree_edge, prev_target_edge, target_edge, sibling_edge, false) );
     }
 
     // 3. Regraft the subtree to the correct edge and set branch lengths
@@ -1262,7 +1308,21 @@ bool sample_SPR_search_one(Parameters& P,MoveStats& Stats, const tree_edge& subt
     if (I.n_attachment_branches() == 1) return false;
 
     // 6. Compute the probabilities for attaching on each branch.
-    spr_attachment_probabilities PrB = SPR_search_attachment_points(P, subtree_edge, range, locations, nodes, sum_out_A);
+    spr_attachment_probabilities PrB;
+    try
+    {
+        P.cache_likelihood_branches();
+	PrB = SPR_search_attachment_points(P, subtree_edge, range, locations, nodes, sum_out_A);
+    }
+    catch (choose_exception<log_double_t>& c)
+    {
+	// FIXME: what if only SOME of the attachment points have NaN/Inf/Inf?
+	std::cerr<<"sample_SPR_search_one: caught choose_exception:\n";
+	std::cerr<<c.what();
+	std::cerr<<"continuing on with P.probability() = "<<P.probability()<<".\n";
+	return false;
+    }
+
 
     vector<log_double_t> Pr = I.convert_to_vector(PrB);
 #ifdef DEBUG_SPR_ALL
