@@ -28,21 +28,25 @@
 /// \author Benjamin Redelings
 /// 
 
+#include "models/path.H"
 #include "mcmc/setup.H"
 #include "mcmc/logger.H"
-#include "util.H"
+#include "util/string/split.H"
+#include "util/range.H"
+#include "util/map.H"
 #include "sample.H"
 #include "alignment/alignment-util.H"
 #include "alignment/alignment-util2.H"
 #include "alignment/alignment-constraint.H"
-#include "timer_stack.H"
-#include "bounds.H"
+#include "util/time.H"
+#include "util/bounds.H"
 #include "AIS.H"
-#include "computation/expression/expression.H"
+#include "computation/expression/bool.H"
+#include "computation/expression/constructor.H"
 
 using boost::program_options::variables_map;
 using boost::dynamic_bitset;
-using boost::optional;
+using std::optional;
 using std::vector;
 using std::endl;
 using std::string;
@@ -50,174 +54,150 @@ using std::ostream;
 using std::map;
 using std::shared_ptr;
 
+template <typename T>
+using Bounds = Box<bounds<T>>;
+
 /// \brief Add a Metropolis-Hastings sub-move for each parameter in \a names to \a M
-void add_modifiable_MH_move(const Model& P, const string& name, const proposal_fn& proposal, int r, const vector<double>& parameters,
-			    MCMC::MoveAll& M, double weight=1)
+void add_modifiable_MH_move(const context_ref& P, const string& name, const proposal_fn& proposal, int rv, const vector<double>& parameters,
+                            MCMC::MoveAll& M, double weight=1)
 {
-    double rate = P.get_rate_for_reg(r);
-    M.add(rate * weight, MCMC::MH_Move( Proposal2M(proposal, r, parameters), name) );
+    auto r_mod = P.get_modifiable_reg(rv);
+    M.add(weight, MCMC::MH_Move( Proposal2M(proposal, *r_mod, parameters), name) );
 }
 
 /// \brief Add a Metropolis-Hastings sub-move for each parameter in \a names to \a M
-void add_MH_move(Model& P, const proposal_fn& proposal, const vector<string>& names,
-		 const vector<string>& pnames, const vector<double>& pvalues,
-		 MCMC::MoveAll& M, double weight=1)
+void add_random_variable_MH_move(const context_ref& P, const string& name, const proposal_fn& proposal, int rv, const vector<double>& parameters,
+				 MCMC::MoveAll& M, double weight=1)
 {
-    // FIXME - Change names to vector<vector<string>>, and deduce a move name from pathname shared by all parameters.
-    // FIXME - then replace the routine above with this one!
-
-    // 1. Set the parameter values to their defaults, if they are not set yet.
-    if (pnames.size() != pvalues.size()) std::abort();
-
-    for(int i=0;i<pnames.size();i++)
-	set_if_undef(*P.keys.modify(), pnames[i], json(pvalues[i]));
-
-    // 2. For each MCMC parameter, create a move for it.
-    for(const auto& parameter_name: names) 
-    {
-	assert(P.maybe_find_parameter(parameter_name));
-
-	Proposal2 proposal2(proposal, parameter_name, pnames, P);
-
-	M.add(weight, MCMC::MH_Move(proposal2, "MH_sample_"+parameter_name));
-    }
+    double rate = P.get_rate_for_random_variable(rv);
+    add_modifiable_MH_move(P, name, proposal, rv, parameters, M, rate * weight);
 }
 
-/// \brief Add a Metropolis-Hastings sub-move for parameter name to M
-///
-/// \param P       The model that contains the parameters
-/// \param p       The proposal function
-/// \param name    The name of the parameter to create a move for
-/// \param pname   The name of the proposal width for this move
-/// \param sigma   A default proposal width, in case the user didn't specify one
-/// \param M       The group of moves to which to add the newly-created sub-move
-/// \param weight  How often to run this move.
-///
-void add_MH_move(Model& P,const proposal_fn& proposal, const string& name, const string& pname,double sigma,
-		 MCMC::MoveAll& M,double weight=1)
-{
-    vector<int> indices = flatten( parameters_with_extension(P,name) );
-    vector<string> names;
-    for(int i: indices)
-	names.push_back(P.parameter_name(i));
-
-    add_MH_move(P, proposal, names, {pname}, {sigma}, M, weight);
-}
-
-double default_sampling_rate(const Model& /*M*/, const string& /*parameter_name*/)
+double default_sampling_rate(const context_ref& /*M*/, const string& /*parameter_name*/)
 {
     return 1.0;
 }
 
-bool add_slice_move(const Model& P, int r, MCMC::MoveAll& M, double weight = 1.0)
+bool add_modifiable_slice_move(int rv, const Bounds<double>& bounds, MCMC::MoveAll& M, double weight = 1.0)
 {
-    double rate = P.get_rate_for_reg(r);
-    auto range = P.get_range_for_reg(r);
-    if (not range.is_a<Bounds<double>>()) return false;
+    string name = "m_real_"+convertToString<int>(rv);
 
-    auto& bounds = range.as_<Bounds<double>>();
-    string name = "m_real_"+convertToString<int>(r);
-    M.add( rate * weight, MCMC::Modifiable_Slice_Move(name, r, bounds, 1.0) );
+    M.add( weight, MCMC::Random_Variable_Slice_Move(name, rv, bounds, 1.0) );
     return true;
 }
 
-void add_boolean_MH_moves(const Model& P, MCMC::MoveAll& M, double weight = 1.0)
+bool add_random_variable_slice_move(const context_ref& P, int rv, MCMC::MoveAll& M, double weight = 1.0)
 {
-    for(int r: P.random_modifiables())
+    double rate = P.get_rate_for_random_variable(rv);
+    auto range = P.get_range_for_random_variable(rv);
+    if (not range.is_a<Bounds<double>>()) return false;
+
+    auto& bounds = range.as_<Bounds<double>>();
+    string name = "m_real_"+convertToString<int>(rv);
+
+    return add_modifiable_slice_move(rv, bounds, M, weight * rate);
+}
+
+void add_boolean_MH_moves(const context_ref& P, MCMC::MoveAll& M, double weight = 1.0)
+{
+    for(int rv: P.random_variables())
     {
-	auto range = P.get_range_for_reg(r);
+	auto range = P.get_range_for_random_variable(rv);
 
 	if (not range.head().is_a<constructor>()) continue;
 	if (range.head().as_<constructor>().f_name != "TrueFalseRange") continue;
 
-	string name = "m_bool_flip_"+convertToString<int>(r);
-	add_modifiable_MH_move(P,name, bit_flip, r, vector<double>{}, M, weight);
+	string name = "m_bool_flip_"+convertToString<int>(rv);
+	add_random_variable_MH_move(P,name, bit_flip, rv, vector<double>{}, M, weight);
     }
 }
 
 /// Find parameters with distribution name Dist
-void add_real_slice_moves(const Model& P, MCMC::MoveAll& M, double weight = 1.0)
+void add_real_slice_moves(const context_ref& P, MCMC::MoveAll& M, double weight = 1.0)
 {
-    for(int r: P.random_modifiables())
-	add_slice_move(P, r, M, weight);
+    for(int rv: P.random_variables())
+	add_random_variable_slice_move(P, rv, M, weight);
 }
 
 /// Find parameters with distribution name Dist
-void add_real_MH_moves(const Model& P, MCMC::MoveAll& M, double weight = 1.0)
+void add_real_MH_moves(const context_ref& P, MCMC::MoveAll& M, double weight = 1.0)
 {
-    for(int r: P.random_modifiables())
+    for(int rv: P.random_variables())
     {
-	auto range = P.get_range_for_reg(r);
+	auto range = P.get_range_for_random_variable(rv);
 	if (not range.is_a<Bounds<double>>()) continue;
 
 	auto& bounds = range.as_<Bounds<double>>();
-	string name = "m_real_cauchy_"+convertToString<int>(r);
-	if (bounds.has_lower_bound and bounds.lower_bound >= 0.0)
-	    add_modifiable_MH_move(P,name, Reflect(bounds, log_scaled(Between(-20,20,shift_cauchy))), r, {1.0}, M, weight);
+	string name = "m_real_cauchy_"+convertToString<int>(rv);
+	if (bounds.lower_bound and *bounds.lower_bound >= 0.0)
+	    add_random_variable_MH_move(P,name, Reflect(bounds, log_scaled(Between(-20,20,shift_cauchy))), rv, {1.0}, M, weight);
 	else
-	    add_modifiable_MH_move(P,name, Reflect(bounds, shift_cauchy), r, {1.0}, M, weight);
+	    add_random_variable_MH_move(P,name, Reflect(bounds, shift_cauchy), rv, {1.0}, M, weight);
     }
 }
 
 /// Find parameters with distribution name Dist
-void add_integer_uniform_MH_moves(const Model& P, MCMC::MoveAll& M, double weight)
+void add_integer_uniform_MH_moves(const context_ref& P, MCMC::MoveAll& M, double weight)
 {
-    for(int r: P.random_modifiables())
+    for(int rv: P.random_variables())
     {
-	auto range = P.get_range_for_reg(r);
+	auto range = P.get_range_for_random_variable(rv);
 	if (not range.is_a<Bounds<int>>()) continue;
 
 	auto& bounds = range.as_<Bounds<int>>();
-	if (not bounds.has_lower_bound or not bounds.has_upper_bound) continue;
-	string name = "m_int_uniform_"+convertToString<int>(r);
-	double l = (int)bounds.lower_bound;
-	double u = (int)bounds.upper_bound;
-	add_modifiable_MH_move(P,name, discrete_uniform_avoid, r, {double(l),double(u)}, M, weight);
+	if (not bounds.lower_bound or not bounds.upper_bound) continue;
+	string name = "m_int_uniform_"+convertToString<int>(rv);
+	double l = *bounds.lower_bound;
+	double u = *bounds.upper_bound;
+	add_random_variable_MH_move(P,name, discrete_uniform_avoid, rv, {double(l),double(u)}, M, weight);
     }
 }
 
-void add_integer_slice_moves(const Model& P, MCMC::MoveAll& M, double weight)
+void add_integer_slice_moves(const context_ref& P, MCMC::MoveAll& M, double weight)
 {
-    for(int r: P.random_modifiables())
+    for(int rv: P.random_variables())
     {
-	auto range = P.get_range_for_reg(r);
-	double rate = P.get_rate_for_reg(r);
+	auto range = P.get_range_for_random_variable(rv);
+	double rate = P.get_rate_for_random_variable(rv);
 	if (not range.is_a<Bounds<int>>()) continue;
 
 	// FIXME: righteousness.
 	// We need a more intelligent way of determining when we should do this.
 	// For example, we do not want to do this for categorical-type variables.
 	auto& bounds = range.as_<Bounds<int>>();
-	if (bounds.has_upper_bound and bounds.has_lower_bound) continue;
+	if (bounds.upper_bound and bounds.lower_bound) continue;
 
-	string name = "m_int_"+convertToString<int>(r);
-	M.add( rate * weight, MCMC::Integer_Modifiable_Slice_Move(name, r, bounds) );
+	string name = "m_int_"+convertToString<int>(rv);
+
+	M.add( rate * weight, MCMC::Integer_Random_Variable_Slice_Move(name, rv, bounds) );
     }
 }
 
-optional<int> scale_is_modifiable_reg(const Model& M, int s)
+optional<int> scale_is_modifiable(const context_ref& M, int s)
 {
     auto& P = dynamic_cast<const Parameters&>(M);
-    return P.compute_expression_is_modifiable_reg(P.branch_scale_index(s));
+    return P.branch_scale(s).is_modifiable(M);
 }
 
-bool all_scales_modifiable(const Model& M)
+bool all_scales_modifiable(const context_ref& M)
 {
     auto& P = dynamic_cast<const Parameters&>(M);
 
     for(int s=0;s<P.n_branch_scales();s++)
-	if (not scale_is_modifiable_reg(M,s))
+	if (not scale_is_modifiable(M,s))
 	    return false;
 
     return true;
 }
 
-void add_alignment_and_parameter_moves(MCMC::MoveAll& moves, Model& M, double weight = 1.0, double enabled = true)
+void add_alignment_and_parameter_moves(MCMC::MoveAll& /*moves*/, context_ref& M, double /*weight*/ = 1.0, double /*enabled*/ = true)
 {
     if (not dynamic_cast<const Parameters*>(&M)) return;
-
+    /* 
     int n = dynamic_cast<const Parameters&>(M).n_imodels();
 
+    // FIXME -- reintroduce realigning everything while proposing e.g. rs07:log_rate or rs07:mean_length
+
     for(int i=0; i<n; i++)
     {
 	string prefix = "I"+convertToString(i+1);
@@ -227,17 +207,17 @@ void add_alignment_and_parameter_moves(MCMC::MoveAll& moves, Model& M, double we
 	string pname1 = model_path({prefix,"rs07:log_rate"});
 	if (auto index = M.maybe_find_parameter(pname1))
 	{
-	    auto proposal = [index,partitions](Model& P){ return realign_and_propose_parameter(P, *index, partitions, shift_cauchy, {0.25}) ;};
+	    auto proposal = [index,partitions](context_ref& P){ return realign_and_propose_parameter(P, *index, partitions, shift_cauchy, {0.25}) ;};
 
-	    moves.add(weight, MCMC::MH_Move(Generic_Proposal(proposal),"realign_and_sample_"+pname1), enabled);
+	    moves.add(weight, MCMC::MH_Move(proposal,"realign_and_sample_"+pname1), enabled);
 	}
 
 	string pname2 = model_path({prefix,"rs07:mean_length"});
 	if (auto index = M.maybe_find_parameter(pname2))
 	{
-	    auto proposal = [index,partitions](Model& P){ return realign_and_propose_parameter(P, *index, partitions, log_scaled(more_than(0.0, shift_laplace)), {0.5}) ;};
+	    auto proposal = [index,partitions](context_ref& P){ return realign_and_propose_parameter(P, *index, partitions, log_scaled(more_than(0.0, shift_laplace)), {0.5}) ;};
 
-	    moves.add(weight, MCMC::MH_Move(Generic_Proposal(proposal),"realign_and_sample_"+pname2), enabled);
+	    moves.add(weight, MCMC::MH_Move(proposal,"realign_and_sample_"+pname2), enabled);
 	}
     }
 
@@ -246,16 +226,16 @@ void add_alignment_and_parameter_moves(MCMC::MoveAll& moves, Model& M, double we
     {
 	string pname = "Scale["+std::to_string(s+1)+"]";
 	vector<int> partitions = dynamic_cast<const Parameters&>(M).partitions_for_scale(s);
-
-	if (auto r = dynamic_cast<const Parameters&>(M).branch_scale_modifiable_reg(s))
+	if (auto r = dynamic_cast<const Parameters&>(M).branch_scale(s).is_modifiable(M))
 	{
-	    auto proposal = [r,partitions](Model& P){
+	    auto proposal = [r,partitions](context_ref& P){
 		return realign_and_propose_parameter(P, *r, partitions, log_scaled(more_than(0.0, shift_laplace)), {0.25}) ;
 	    };
 
-	    moves.add(weight, MCMC::MH_Move(Generic_Proposal(proposal),"realign_and_sample_"+pname), enabled);
+	    moves.add(weight, MCMC::MH_Move(proposal,"realign_and_sample_"+pname), enabled);
 	}
     }
+    */
 }
 
 //FIXME - how to make a number of variants with certain things fixed, for burn-in?
@@ -267,13 +247,13 @@ void add_alignment_and_parameter_moves(MCMC::MoveAll& moves, Model& M, double we
 ///
 /// \param P   The model and state.
 ///
-MCMC::MoveAll get_parameter_MH_moves(Model& M)
+MCMC::MoveAll get_parameter_MH_moves(context_ref& M)
 {
     MCMC::MoveAll MH_moves("parameters:MH");
 
     if (Parameters* P = dynamic_cast<Parameters*>(&M))
 	for(int i=0;i<P->n_branch_scales();i++)
-	    if (auto r = scale_is_modifiable_reg(M, i))
+	    if (auto r = scale_is_modifiable(M, i))
 	    {
 		string name = "scale_scale_"+convertToString<int>(i+1);
 		add_modifiable_MH_move(M, name, log_scaled(Between(-20,20,shift_cauchy)),    *r, {1.0}, MH_moves);
@@ -304,14 +284,15 @@ MCMC::MoveAll get_parameter_MH_moves(Model& M)
       add_MH_move(P, log_scaled(Between(-20,20,shift_cauchy)),    "*.gamma.sigma/mu","gamma.sigma_scale_sigma",  0.25, MH_moves);
       add_MH_move(P, log_scaled(Between(-20,0,shift_cauchy)),    "*.Beta.varOverMu", "beta.Var_scale_sigma",  0.25, MH_moves);
       add_MH_move(P, log_scaled(Between(-20,20,shift_cauchy)),    "*.LogNormal.sigma_over_mu","log-normal.sigma_scale_sigma",  0.25, MH_moves);
+
+      add_MH_move(M, shift_delta,                  "b*.delta",       "lambda_shift_sigma",     0.35, MH_moves, 10);
+      add_MH_move(M, Between(-20,20,shift_cauchy),  model_path({"*","rs07:log_rate"}),      "lambda_shift_sigma",    0.25, MH_moves, 10);
+      add_MH_move(M, log_scaled(more_than(0.0, shift_laplace)), model_path({"*","rs07:mean_length"}), "length_shift_sigma",   0.5, MH_moves, 10);
+
     */
     if (dynamic_cast<Parameters*>(&M) and all_scales_modifiable(M))
 	MH_moves.add(4,MCMC::SingleMove(scale_means_only, "scale_Scales_only", "scale"));
   
-    add_MH_move(M, shift_delta,                  "b*.delta",       "lambda_shift_sigma",     0.35, MH_moves, 10);
-    add_MH_move(M, Between(-20,20,shift_cauchy),  model_path({"*","rs07:log_rate"}),      "lambda_shift_sigma",    0.25, MH_moves, 10);
-    add_MH_move(M, log_scaled(more_than(0.0, shift_laplace)), model_path({"*","rs07:mean_length"}), "length_shift_sigma",   0.5, MH_moves, 10);
-
     // Add these moves disabled
     add_alignment_and_parameter_moves(MH_moves, M, 1.0, false);
 
@@ -322,8 +303,8 @@ MCMC::MoveAll get_scale_slice_moves(Parameters& P)
 {
     MCMC::MoveAll slice_moves("parameters:scale:MH");
     for(int i=0;i<P.n_branch_scales();i++)
-	if (auto r = scale_is_modifiable_reg(P,i))
-	    add_slice_move(P, *r, slice_moves);
+	if (auto r = scale_is_modifiable(P,i))
+	    add_modifiable_slice_move(*r, lower_bound<double>(0), slice_moves);
 
     return slice_moves;
 }
@@ -332,7 +313,7 @@ MCMC::MoveAll get_scale_slice_moves(Parameters& P)
 ///
 /// \param P   The model and state.
 ///
-MCMC::MoveAll get_parameter_slice_moves(Model& M)
+MCMC::MoveAll get_parameter_slice_moves(context_ref& M)
 {
     MCMC::MoveAll slice_moves("parameters:slice");
 
@@ -340,8 +321,8 @@ MCMC::MoveAll get_parameter_slice_moves(Model& M)
     {
 	// scale parameters - do we need this?
 	for(int i=0;i<P->n_branch_scales();i++)
-	    if (auto r = scale_is_modifiable_reg(M,i))
-		add_slice_move(*P, *r, slice_moves);
+	    if (auto r = scale_is_modifiable(M,i))
+		add_modifiable_slice_move(*r, lower_bound<double>(0), slice_moves);
 
 	if (all_scales_modifiable(M))
 	    slice_moves.add(2,MCMC::Scale_Means_Only_Slice_Move("scale_Scales_only_slice",0.6));
@@ -420,18 +401,6 @@ MCMC::MoveAll get_alignment_moves(Parameters& P)
     return alignment_moves;
 }
 
-MCMC::MoveAll get_h_moves(Model& M)
-{
-    using namespace MCMC;
-
-    MoveAll h_moves("haskell_moves");
-
-    for(int i=0;i<M.n_transition_kernels();i++)
-	h_moves.add(1,IOMove(i));
-
-    return h_moves;
-}
-
 /// \brief Construct moves to sample the tree
 ///
 /// \param P   The model and state.
@@ -545,7 +514,7 @@ MCMC::MoveAll get_tree_moves(Parameters& P)
 ///
 /// \param P   The model and state.
 ///
-MCMC::MoveAll get_parameter_MH_but_no_slice_moves(Model& M)
+MCMC::MoveAll get_parameter_MH_but_no_slice_moves(context_ref& M)
 {
     using namespace MCMC;
 
@@ -563,36 +532,9 @@ MCMC::MoveAll get_parameter_MH_but_no_slice_moves(Model& M)
     // Actually there ARE slice moves for this, but they don't jump modes!
     add_real_MH_moves(M, parameter_moves, 0.1);
 
-    // FIXME - we need a proposal that sorts after changing
-    //         then we can un-hack the recalc function in smodel.C
-    //         BUT, this assumes that we have the DP::rate* names in *numerical* order
-    //          whereas we probably find them in *lexical* order....
-    //          ... or creation order?  That might be OK for now! 
-
-    for(int i=0;;i++) {
-	string name = "M3.omega" + convertToString(i+1);
-	if (not has_parameter(M,name))
-	    break;
-    
-	add_MH_move(M, log_scaled(Between(-20,20,shift_cauchy)), name, "omega_scale_sigma", 1, parameter_moves);
-	//    Proposal2 m(log_scaled(shift_cauchy), name, vector<string>(1,"omega_scale_sigma"), P);
-	//    parameter_moves.add(1, MCMC::MH_Move(m,"sample_M3.omega"));
-    }
-
+    if (auto MM = dynamic_cast<Model*>(&M); MM->contains_key("sample_foreground_branch"))
     {
-	auto index = M.maybe_find_parameter("lambdaScaleBranch");
-	if (index and (M.get_parameter_value(*index).as_int() != -1 or M.contains_key("lambda_search_all")))
-	{
-	    M.set_parameter_value(*index, 0);
-	    Generic_Proposal m(move_scale_branch);
-	    parameter_moves.add(1.0, MCMC::MH_Move(m,"sample_lambdaScaleBranch"));
-	}
-    }
-
-    if (M.contains_key("sample_foreground_branch"))
-    {
-	Generic_Proposal m(move_subst_type_branch);
-	parameter_moves.add(1.0, MCMC::MH_Move(m,"sample_foreground_branch"));
+	parameter_moves.add(1.0, MCMC::MH_Move(move_subst_type_branch,"sample_foreground_branch"));
     }
 
     return parameter_moves;
@@ -662,7 +604,7 @@ double fraction_non_gap(const Parameters& P)
     return total_char/total_cell;
 }
 
-void log_preburnin(ostream& o, const Model& M, const string& name, int iter)
+void log_preburnin(ostream& o, const context_ref& M, const string& name, int iter)
 {
     auto& P = dynamic_cast<const Parameters&>(M);
 
@@ -671,7 +613,7 @@ void log_preburnin(ostream& o, const Model& M, const string& name, int iter)
     o<<"   |T| = "<<TL;
     o<<"   |A| = "<<MCMC::alignment_length(P);
     for(int s=0;s<P.n_branch_scales();s++)
-	o<<"   Scale"<<s+1<<"*|T| = "<<P.branch_scale(s)*TL;
+	o<<"   Scale"<<s+1<<"*|T| = "<<P.get_branch_scale(s)*TL;
     o<<std::endl;
 }
 
@@ -718,18 +660,18 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	    // turn training on
 	    {
 		Parameters& PP = *P.as<Parameters>();
-		PP.set_parameter_value(PP.find_parameter("*IModels.training"), bool_true);
+		PP.set_imodel_training(true);
 	    }
 
 	    pre_burnin.iterate(P,Stats);
 	    // turn training off
 	    {
 		Parameters& PP = *P.as<Parameters>();
-		PP.set_parameter_value(PP.find_parameter("*IModels.training"), bool_false);
+		PP.set_imodel_training(false);
 	    }
 
 	    log_preburnin(out_both, *P, "(S)+(L)+(P)+Alignment", i);
-	    show_parameters(out_log, *P, false);
+	    show_parameters(out_log, *P);
 	}
 	out_both<<endl;
     }
@@ -751,7 +693,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<n_pre_burnin;i++) {
 	    pre_burnin.iterate(P,Stats);
 	    log_preburnin(out_both, *P, "Tree (S)ize", i);
-	    show_parameters(out_log, *P, false);
+	    show_parameters(out_log, *P);
 	}
     }
     out_both<<endl;
@@ -771,7 +713,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<n_pre_burnin;i++) {
 	    pre_burnin.iterate(P,Stats);
 	    log_preburnin(out_both, *P, "(S)+Branch (L)engths", i);
-	    show_parameters(out_log, *P, false);
+	    show_parameters(out_log, *P);
 	}
     }
     out_both<<endl;
@@ -792,7 +734,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<n_pre_burnin;i++) {
 	    pre_burnin.iterate(P,Stats);
 	    log_preburnin(out_both, *P, "(S)+(L)+(P)arameters", i);
-	    show_parameters(out_log, *P, false);
+	    show_parameters(out_log, *P);
 	}
     }
     out_both<<endl;
@@ -814,7 +756,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<n_pre_burnin2;i++) {
 	    pre_burnin.iterate(P,Stats);
 	    log_preburnin(out_both, *P, "(S)+(L)+(P)+NNI", i);
-	    show_parameters(out_log, *P, false);
+	    show_parameters(out_log, *P);
 	}
     }
     out_both<<endl;
@@ -834,7 +776,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<1;i++) {
 	    pre_burnin.iterate(P,Stats);
 	    log_preburnin(out_both, *P, "SPR", i);
- 	    show_parameters(out_log, *P, false);
+ 	    show_parameters(out_log, *P);
 	}
 	out_both<<endl;
     }
@@ -855,7 +797,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<n_pre_burnin;i++) {
 	    pre_burnin.iterate(P,Stats);
 	    log_preburnin(out_both, *P, "(S)+(L)+(P)+NNI", i);
-	    show_parameters(out_log, *P, false);
+	    show_parameters(out_log, *P);
 	}
     }
     out_both<<endl;
@@ -870,7 +812,7 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	// turn training on
 	{
 	    Parameters& PP = *P.as<Parameters>();
-	    PP.set_parameter_value(PP.find_parameter("*IModels.training"), bool_true);
+	    PP.set_imodel_training(true);
 	}
 
 	MoveAll pre_burnin("pre-burnin+A");
@@ -889,19 +831,19 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 	for(int i=0;i<n_pre_burnin;i++)
 	{
 	    log_preburnin(out_both, *P, "SPR+A (training)", i);
-	    show_parameters(out_log,*P,false);
+	    show_parameters(out_log,*P);
 	    pre_burnin.iterate(P,Stats);
 	}
 
 	// turn training back off
 	{
 	    Parameters& PP = *P.as<Parameters>();
-	    PP.set_parameter_value(PP.find_parameter("*IModels.training"), bool_false);
+	    PP.set_imodel_training(false);
 	}
 	for(int i=0;i<n_pre_burnin;i++)
 	{
 	    log_preburnin(out_both, *P, "SPR+A", i);
-	    show_parameters(out_log,*P,false);
+	    show_parameters(out_log,*P);
 	    pre_burnin.iterate(P,Stats);
 	}
 
@@ -915,8 +857,8 @@ void do_pre_burnin(const variables_map& args, owned_ptr<Model>& P, ostream& out_
 
     int B = P.as<Parameters>()->t().n_branches();
     for(int i=0; i<P.as<Parameters>()->n_branch_scales(); i++)
-	if (P.as<Parameters>()->branch_scale(i) > 0.5*B)
-	    P.as<Parameters>()->branch_scale(i, 0.5*B);
+	if (P.as<Parameters>()->get_branch_scale(i) > 0.5*B)
+	    P.as<Parameters>()->set_branch_scale(i, 0.5*B);
 }
 
 /// \brief Create transition kernels and start a Markov chain
@@ -985,9 +927,6 @@ void do_sampling(const variables_map& args,
     if (P->load_value("slice_sampling", true))
 	sampler.add(1,slice_moves);
 
-    //------------------- Add moves defined via notes ---------------------------//
-    sampler.add(1, get_h_moves(*P));
-
     for(int i=0;i<loggers.size();i++)
 	sampler.add_logger(loggers[i]);
 
