@@ -23,115 +23,165 @@
 /// \brief This file implements classes and functions for uniform slice sampling.
 ///
 
-#include "slice-sampling.H"
-#include "rng.H"
-#include "probability/choose.H"
-#include "computation/expression/expression.H"
 #include "util/assert.hh"
+#include "slice-sampling.H"
+#include "util/rng.H"
+#include "probability/choose.H"
+#include "mcmc/sample-alignment.H"
+
+extern int log_verbose;
 
 using std::vector;
 
-namespace slice_sampling {
-    double identity(double x) {return x;}
-}
-
 double slice_function::current_value() const
 {
     std::abort();
 }
 
 // ********************************** modifiable slice function ****************************************** //
-model_slice_function::model_slice_function(Model& m)
-    :model_slice_function(m, {})
+context_slice_function::context_slice_function(context_ref& c)
+    :context_slice_function(c, {})
 {}
 
-model_slice_function::model_slice_function(Model& m, const Bounds<double>& b)
-    :slice_function(b), C0(m), M(m)
+context_slice_function::context_slice_function(context_ref& c, const bounds<double>& b)
+    :slice_function(b), C0(c), C(c)
 {
     current_fn_value.log() = 0;
 }
 
-double model_slice_function::operator()(double x)
+double context_slice_function::operator()(double x)
 {
     count++;
 
-    // We are intentionally only calling Model::operator==( ) here.
+    // We are intentionally only calling context::operator==( ) here.
     // Maybe we should actually call merely context::operator==( ) though?
-    reinterpret_cast<context&>(M) = C0;
+    if (count == 1) C0.evaluate_program();
+    C = C0;
     set_value(x);
 
     // Here is where we return 0 if the number of variables changes.
     // How can we automate this so that it is called only once?
-    current_fn_value = M.heated_probability_ratio(C0);
+    auto ratio = C.heated_probability_ratios(C0);
+    if (ratio.variables_changed)
+        throw variables_changed_exception("Variable changed during slice sampling!");
+
+    current_fn_value = ratio.total_ratio();
+
     return operator()();
 }
 
-double model_slice_function::operator()()
+double context_slice_function::operator()()
 {
     return log(current_fn_value);
 }
 
-void modifiable_slice_function::set_value(double x)
+void context_slice_function::reset()
 {
-    M.set_modifiable_value(m, x);
+    C = C0;
 }
 
-double modifiable_slice_function::current_value() const
+void random_variable_slice_function::set_value(double x)
 {
-    return M.get_modifiable_value(m).as_double();
+    C.set_modifiable_value(r_mod, x);
 }
 
-modifiable_slice_function::modifiable_slice_function(Model& M_, const Bounds<double>& bounds, int m_)
-    :model_slice_function(M_, bounds), m(m_)
-{ }
-
-// ******************************* integer modifiable slice function *************************************** //
-Bounds<double> convert_bounds(const Bounds<int>& int_bounds)
+double random_variable_slice_function::current_value() const
 {
-    Bounds<double> double_bounds;
+    return C.get_modifiable_value(r_mod).as_double();
+}
 
-    double_bounds.lower_bound = int_bounds.lower_bound;
-    double_bounds.has_lower_bound = int_bounds.has_lower_bound;
+random_variable_slice_function::random_variable_slice_function(context_ref& c, const bounds<double>& bounds, int rv)
+    :context_slice_function(c, bounds)
+{
+    if (auto m = C.get_modifiable_reg(rv))
+	r_mod = *m;
+    else
+	throw myexception()<<"No modifiable reg for slice function!";
+}
 
-    double_bounds.upper_bound = int_bounds.upper_bound+1;
-    double_bounds.has_upper_bound = int_bounds.has_upper_bound;
+// ******************************* integer random_variable slice function *************************************** //
+bounds<double> convert_bounds(const bounds<int>& int_bounds)
+{
+    bounds<double> double_bounds = int_bounds;
+    if (double_bounds.upper_bound)
+        double_bounds.upper_bound = *double_bounds.upper_bound + 1;
 
     return double_bounds;
 }
 
-void integer_modifiable_slice_function::set_value(double x)
+void integer_random_variable_slice_function::set_value(double x)
 {
     int x_integer = (int)floor(x);
-    M.set_modifiable_value(m, x_integer);
+    C.set_modifiable_value(r_mod, x_integer);
 }
 
-double integer_modifiable_slice_function::current_value() const
+double integer_random_variable_slice_function::current_value() const
 {
-    return M.get_modifiable_value(m).as_int();
+    return C.get_modifiable_value(r_mod).as_int();
 }
 
-integer_modifiable_slice_function::integer_modifiable_slice_function(Model& M_, const Bounds<int>& bounds, int m_)
-    :model_slice_function(M_, convert_bounds(bounds)), m(m_)
-{ }
+integer_random_variable_slice_function::integer_random_variable_slice_function(context_ref& c, const bounds<int>& bounds, int rv)
+    :context_slice_function(c, convert_bounds(bounds))
+{
+    if (auto m = C.get_modifiable_reg(rv))
+	r_mod = *m;
+    else
+	throw myexception()<<"No modifiable reg for slice function!";
+}
 
 // ******************************* branch length slice function *************************************** //
 
 void branch_length_slice_function::set_value(double l)
 {
-    static_cast<Parameters&>(M).setlength(b,l);
+    static_cast<Parameters&>(C).setlength(b,l);
 }
 
 double branch_length_slice_function::current_value() const
 {
-    return static_cast<Parameters&>(M).t().branch_length(b);
+    return static_cast<Parameters&>(C).t().branch_length(b);
 }
 
 branch_length_slice_function::branch_length_slice_function(Parameters& P,int b_)
-    :model_slice_function(P),b(b_)
+    :context_slice_function(P),b(b_)
 { 
     set_lower_bound(0);
 }
 
+double alignment_branch_length_slice_function::operator()(double x)
+{
+    count++;
+
+    // We are intentionally only calling context::operator==( ) here.
+    // Maybe we should actually call merely context::operator==( ) though?
+    if (count == 1) C0.evaluate_program();
+    C = C0;
+    set_value(x);
+    auto alignment_sum_ratio_1 = sample_alignment(static_cast<Parameters&>(C),b);
+
+    // Here is where we return 0 if the number of variables changes.
+    // How can we automate this so that it is called only once?
+    current_fn_value = C.heated_probability_ratio(C0)*alignment_sum_ratio_1/alignment_sum_ratio_0;
+    return operator()();
+}
+
+void alignment_branch_length_slice_function::set_value(double l)
+{
+    static_cast<Parameters&>(C).setlength(b,l);
+}
+
+double alignment_branch_length_slice_function::current_value() const
+{
+    return static_cast<Parameters&>(C).t().branch_length(b);
+}
+
+alignment_branch_length_slice_function::alignment_branch_length_slice_function(Parameters& P,int b_)
+    :context_slice_function(P),b(b_)
+{ 
+    set_lower_bound(0);
+    alignment_sum_ratio_0 = sample_alignment(P, b);
+    C0 = P;
+}
+
 void slide_node_slice_function::set_value(double x) 
 {
     /* Problem: If y << x, then we may not be able to recover y = (x+y)-x.
@@ -140,17 +190,17 @@ void slide_node_slice_function::set_value(double x)
        Solution: Store x0 and y0 separately, and compute y = y0 + (x0-x).
     */
     assert(0 <= x and x <= (x0+y0));
-    static_cast<Parameters&>(M).setlength(b1,x);
-    static_cast<Parameters&>(M).setlength(b2,y0+(x0-x));
+    static_cast<Parameters&>(C).setlength(b1,x);
+    static_cast<Parameters&>(C).setlength(b2,y0+(x0-x));
 }
 
 double slide_node_slice_function::current_value() const
 {
-    return static_cast<Parameters&>(M).t().branch_length(b1);
+    return static_cast<Parameters&>(C).t().branch_length(b1);
 }
 
 slide_node_slice_function::slide_node_slice_function(Parameters& P,int b0)
-    :model_slice_function(P)
+    :context_slice_function(P)
 {
     vector<int> b = P.t().branches_after(b0);
 
@@ -168,7 +218,7 @@ slide_node_slice_function::slide_node_slice_function(Parameters& P,int b0)
 }
 
 slide_node_slice_function::slide_node_slice_function(Parameters& P,int i1,int i2)
-    :model_slice_function(P), b1(i1), b2(i2)
+    :context_slice_function(P), b1(i1), b2(i2)
 {
     x0 = P.t().branch_length(b1);
     y0 = P.t().branch_length(b2);
@@ -177,161 +227,87 @@ slide_node_slice_function::slide_node_slice_function(Parameters& P,int i1,int i2
     set_upper_bound(x0+y0);
 }
 
-/// \brief Compute the sum of the branch mean parameters for \a P
-double sum_of_means(const Parameters& P)
-{
-    double sum = 0;
-    for(int i=0;i<P.n_branch_scales();i++) 
-	sum += P.branch_scale(i);
-    return sum;
-}
 
-/// \brief Scale the branch means of \a P so that they sum to \a t, but do not invalidate cached values.
-///
-/// \param P The model state to modify.
-/// \param t The sum of the branch means after modification.
-///
-/// The lengths D=mu*T are not updated by calling recalc, so that the smodel and imodel do not get updated.
-///
-double set_sum_of_means(Parameters& P, double t)
-{
-    double sum = sum_of_means(P);
-    double scale = t/sum;
-    for(int i=0;i<P.n_branch_scales();i++) 
-	P.branch_scale(i,P.branch_scale(i) * scale);
-
-    return scale;
-}
+/*
+ * Joint scaling of branch lengths and scales so that the T*R=D remains constant.
+ */
 
 void scale_means_only_slice_function::set_value(double t)
 {
-    auto& P = static_cast<Parameters&>(M);
+    log_current_factor = t;
+    double scale = exp(log_current_factor);
 
-    // Set the values of \mu[i]
-    double scale = set_sum_of_means(P, initial_sum_of_means * exp(t));
+    auto& P = static_cast<Parameters&>(C);
 
-    // Scale the tree in the opposite direction
-    for(int b=0;b<P.t().n_branches();b++) 
-    {
-	const double L = P.t().branch_length(b);
-	P.setlength_unsafe(b, L/scale);
-    }
+    // Scale the branch-scaling factor for each partition.
+    for(int i=0; i<initial_scales.size(); i++)
+	P.set_branch_scale(i, initial_scales[i]*scale);
+
+    // Scale the branch lengths in the opposite direction
+    for(int b=0; b<initial_branch_lengths.size(); b++)
+	P.setlength_unsafe(b, initial_branch_lengths[b]/scale);
+}
+
+double scale_means_only_slice_function::log_average_scale() const
+{
+    return log(initial_average_scale) + log_current_factor;
 }
 
 double scale_means_only_slice_function::operator()()
 {
-    auto& P = static_cast<Parameters&>(M);
-
-    const int B = P.t().n_branches();
-    const int n = P.n_branch_scales();
-
     // return pi * (\sum_i \mu_i)^(n-B)
-    return model_slice_function::operator()() + log(sum_of_means(P))*(n-B);
+    return context_slice_function::operator()() + log_average_scale()*((int)initial_scales.size() - (int)initial_branch_lengths.size());
 }
 
 double scale_means_only_slice_function::current_value() const
 {
-    auto& P = static_cast<Parameters&>(M);
-    double t = log(sum_of_means(P)/initial_sum_of_means);
-    return t;
+    return log_current_factor;
 }
 
 scale_means_only_slice_function::scale_means_only_slice_function(Parameters& P)
-    :model_slice_function(P),
-     initial_sum_of_means(sum_of_means(P))
+    :context_slice_function(P)
 { 
-    Bounds<double>& b = *this;
+    // 1. Set up initial scales
+    if (P.n_branch_scales() == 0)
+        throw myexception()<<"Can't do scale_means_only_slice function if there are no scales!";
 
-#ifndef NDEBUG
-    std::clog<<"Bounds on t are "<<b<<std::endl;
-#endif
-
-    for(int i=0; i<P.n_branch_scales(); i++)
+    initial_average_scale = 0;
+    initial_scales.resize(P.n_branch_scales());
+    for(int i=0;i<initial_scales.size();i++)
     {
-	Bounds<double> b2 = P.get_bounds_for_compute_expression(P.branch_scale_index(i));
-
-	if (b2.has_lower_bound and b2.lower_bound > 0)
-	{
-	    b2.has_lower_bound = true;
-	    b2.lower_bound = log(b2.lower_bound) - log(P.branch_scale(i));
-	}
-	else
-	    b2.has_lower_bound = false;
-
-	if (b2.has_upper_bound)
-	    b2.upper_bound = log(b2.upper_bound) - log(P.branch_scale(i));
-
-	b = b and b2;
+        initial_scales[i] = P.get_branch_scale(i);
+        initial_average_scale += initial_scales[i];
     }
+    initial_average_scale /= initial_scales.size();
+
+    // FIXME: We should be able to assert that all of the scales are modifiable.
+
+    // 3. Set up initial branch_lengths
+    initial_branch_lengths.resize(P.t().n_branches());
+    for(int b=0; b<initial_branch_lengths.size(); b++)
+        initial_branch_lengths[b] = P.t().branch_length(b);
+
+    // FIXME: We should be able to assert that all of the branch lengths are modifiable.
+
+    // 4. Set bounds on the scale factor to avoid numeric overflow/underflow.
+    bounds<double>& b = *this;
+
+    // We want to require that
+    //     log(average_scale)                              \in [-40,40]
+    //     log(initial_average_scale) + log_current_factor \in [-40,40]
+    //                                  log_current_factor \in [-40 - log(initial_average_scale), 40 - log(initial_average_scale)]
+    double shift = -log(initial_average_scale);
+
+    b = between<double>(-40+shift,40+shift);
 
 #ifndef NDEBUG
-    std::clog<<"Bounds on t are now "<<b<<std::endl<<std::endl;
+    std::clog<<"bounds on t are "<<b<<std::endl;
 #endif
 }
 
-void constant_sum_slice_function::set_value(double t)
-{
-    auto& P = static_cast<Parameters&>(M);
-    vector<expression_ref> y = P.get_parameter_values(indices);
-    vector<double> x = vec_to_double(y);
-
-    double total = sum(x);
-
-    double factor = (total - t)/(total - x[n]);
-
-    for(int i=0;i<indices.size();i++)
-	if (i == n)
-	    x[i] = t;
-	else
-	    x[i] *= factor;
-
-    assert(std::abs(::sum(x) - total) < 1.0e-9);
-
-    for(int i=0;i<y.size();i++)
-	y[i] = x[i];
-
-    P.set_parameter_values(indices,y);
-}
-
-
-double constant_sum_slice_function::operator()()
-{
-    auto& P = static_cast<Parameters&>(M);
-
-    vector<double> x = vec_to_double(P.get_parameter_values(indices));
-
-    double total = sum(x);
-
-    double t = current_value();
-
-    const int N = indices.size();
-
-    // return pi * (1-x)^(N-1)
-    return model_slice_function::operator()() + (N-1)*log(total-t);
-}
-
-double constant_sum_slice_function::current_value() const
-{
-    auto& P = static_cast<Parameters&>(M);
-    return P.get_parameter_value(indices[n]).as_double();
-}
-
-
-constant_sum_slice_function::constant_sum_slice_function(Model& P, const vector<int>& indices_,int n_)
-    :model_slice_function(P),
-     indices(indices_),
-     n(n_)
-{ 
-    vector<double> x = vec_to_double(P.get_parameter_values(indices));
-    double total = sum(x);
-
-    set_lower_bound(0);
-    set_upper_bound(total);
-}
-
 void constant_sum_modifiable_slice_function::set_value(double t)
 {
-    auto& P = static_cast<Parameters&>(M);
+    auto& P = static_cast<Parameters&>(C);
     const int N = indices.size();
 
     vector<double> x(N);
@@ -357,7 +333,7 @@ void constant_sum_modifiable_slice_function::set_value(double t)
 
 double constant_sum_modifiable_slice_function::operator()()
 {
-    auto& P = static_cast<Parameters&>(M);
+    auto& P = static_cast<Parameters&>(C);
     const int N = indices.size();
 
     double total = 0;
@@ -367,21 +343,21 @@ double constant_sum_modifiable_slice_function::operator()()
     double t = current_value();
 
     // return pi * (1-x)^(N-1)
-    return model_slice_function::operator()() + (N-1)*log(total-t);
+    return context_slice_function::operator()() + (N-1)*log(total-t);
 }
 
 double constant_sum_modifiable_slice_function::current_value() const
 {
-    auto& P = static_cast<Parameters&>(M);
+    auto& P = static_cast<Parameters&>(C);
     return P.get_modifiable_value(indices[n]).as_double();
 }
 
-constant_sum_modifiable_slice_function::constant_sum_modifiable_slice_function(Model& M, const vector<int>& indices_,int n_)
-    :model_slice_function(M),
+constant_sum_modifiable_slice_function::constant_sum_modifiable_slice_function(context_ref& c, const vector<int>& indices_,int n_)
+    :context_slice_function(c),
      indices(indices_),
      n(n_)
 { 
-    vector<double> x = vec_to_double(M.get_modifiable_values(indices));
+    auto x = (vector<double>)C.get_modifiable_values(indices);
     double total = sum(x);
 
     set_lower_bound(0);
@@ -434,8 +410,8 @@ find_slice_boundaries_stepping_out(double x0,slice_function& g,double logy, doub
 
     // Shrink interval to lower and upper bounds.
 
-    if (g.below_lower_bound(L)) L = g.lower_bound;
-    if (g.above_upper_bound(R)) R = g.upper_bound;
+    if (g.below_lower_bound(L)) L = *g.lower_bound;
+    if (g.above_upper_bound(R)) R = *g.upper_bound;
 
     assert(L < R);
 
@@ -484,8 +460,15 @@ double search_interval(double x0,double& L, double& R, slice_function& g,double
     return x0;
 }
 
-double slice_sample(double x0, slice_function& g,double w, int m)
+double slice_sample_(double x0, slice_function& g, double w, int m)
 {
+    // If x is not in the range then this could be a range that is reduced to avoid loss of precision.
+    if (not g.in_range(x0))
+    {
+        if (log_verbose >= 4) std::cerr<<x0<<" not in range!";
+        return x0;
+    }
+
     assert(g.in_range(x0));
 
     double gx0 = g();
@@ -515,6 +498,20 @@ double slice_sample(double x0, slice_function& g,double w, int m)
     return search_interval(x0,L,R,g,logy);
 }
 
+double slice_sample(double x0, slice_function& g,double w, int m)
+{
+    try
+    {
+        return slice_sample_(x0, g, w, m);
+    }
+    catch (variables_changed_exception& e)
+    {
+        if (log_verbose >= 3) std::cerr<<e.what()<<"\n";
+        g.reset();
+    }
+    return x0;
+}
+
 double slice_sample(slice_function& g, double w, int m)
 {
     double x0 = g.current_value();
@@ -525,7 +522,7 @@ inline static void dec(double& x, double w, const slice_function& g, bool& hit_l
 {
     x -= w;
     if (g.below_lower_bound(x)) {
-	x = g.lower_bound;
+	x = *g.lower_bound;
 	hit_lower_bound=true;
     }
 }
@@ -534,7 +531,7 @@ inline static void inc(double& x, double w, const slice_function& g, bool& hit_u
 {
     x += w;
     if (g.above_upper_bound(x)) {
-	x = g.upper_bound;
+	x = *g.upper_bound;
 	hit_upper_bound = true;
     }
 }
