@@ -3,8 +3,9 @@
 #include <list>
 #include "models/parse.H"
 #include "models/model.H"
-#include "myexception.H"
-#include "util.H"
+#include "util/myexception.H"
+#include "util/string/join.H"
+#include "util/range.H" // for reverse( )
 #include "rules.H"
 #include "setup.H"
 
@@ -12,17 +13,17 @@ using std::vector;
 using std::list;
 using std::string;
 using std::pair;
-using boost::optional;
+using std::optional;
 
 optional<pair<string,string>> split_keyword(const string& s, char c)
 {
     for(int i=0;i<s.size();i++)
     {
-	if (s[i] == '[' or s[i] == ']') return boost::none;
+	if (s[i] == '[' or s[i] == ']') return {};
 	if (s[i] == c)
 	    return pair<string,string>({s.substr(0,i),s.substr(i+1)});
     }
-    return boost::none;
+    return {};
 }
 
 /// \brief Turn an expression of the form h1[a]+h2[b] -> {h1[a],h2[b]}
@@ -76,9 +77,10 @@ vector<string> split_args(string s)
 	return args;
     }
 
-    if (pos == 0) throw myexception()<<"Term has no operator!";
-
-    args = { s.substr(0,pos) };
+    if (pos == 0)
+        args = {"List"};
+    else
+        args = { s.substr(0,pos) };
     s = s.substr(pos);
 
     //2. Get the arguments from '[arg1, arg2, ... , argn]'.
@@ -385,7 +387,7 @@ optional<list<ptree>> get_list_elements(const ptree& p)
 	    return xs;
 	}
     }
-    return boost::none;
+    return {};
 }
 
 optional<list<ptree>> get_ann_list_elements(const ptree& ann)
@@ -406,7 +408,7 @@ optional<list<ptree>> get_ann_list_elements(const ptree& ann)
 	    return xs;
 	}
     }
-    return boost::none;
+    return {};
 }
 
 string unparse(const ptree& p, const Rules& rules)
@@ -587,7 +589,7 @@ optional<ptree> peel_sample(ptree p)
     if (p.has_value<string>() and p.get_value<string>() == "sample")
 	return p[0].second;
     else
-	return boost::none;
+	return {};
 }
 
 optional<ptree> peel_sample_annotated(ptree p)
@@ -596,7 +598,7 @@ optional<ptree> peel_sample_annotated(ptree p)
     if (v.has_value<string>() and v.get_value<string>() == "sample")
 	return v[0].second;
     else
-	return boost::none;
+	return {};
 }
 
 string unparse_abbrev(ptree p, const Rules& rules, int length)
@@ -650,49 +652,3 @@ bool is_constant(const ptree& model)
 
     return (name.size()>=2 and name[0] == '"' and name.back() == '"');
 }
-
-string translate_structures(const string& name)
-{
-    vector<string> path = model_split_path(name);
-
-    vector<string> path2;
-    optional<int> elem;
-    for(auto& x: path)
-    {
-	if (x == "Cons:second")
-	{
-	    if (elem)
-		elem = 1+*elem;
-	    else
-		elem = 1;
-	}
-	else if (x == "Cons:first")
-	{
-	    if (path2.empty()) path2.push_back("");
-	    if (not elem) elem = 0;
-	    elem = 1+*elem;
-
-	    path2.back() += "[" + convertToString(*elem) + "]";
-	    elem = boost::none;
-	}
-	else
-	{
-	    elem = boost::none;
-	    if (x == "Pair::first")
-	    {
-		if (path2.empty()) path2.push_back("");
-		path2.back() += "[1]";
-	    }
-	    else if (x == "Pair::second")
-	    {
-		if (path2.empty()) path2.push_back("");
-		path2.back() += "[2]";
-	    }
-	    else
-		path2.push_back(x);
-	}
-    }
-    return model_path(path2);
-}
-
-
