@@ -1,12 +1,13 @@
-#include "ptree.H"
-#include "util.H"
-#include "myexception.H"
+#include "util/ptree.H"
+#include "util/string/join.H"
+#include "util/string/convert.H"
+#include "util/myexception.H"
 
 using std::string;
 using std::vector;
 using std::pair;
 
-using boost::optional;
+using std::optional;
 
 std::ostream& operator<<(std::ostream& o,const monostate&) {o<<"()";return o;}
 
@@ -34,7 +35,7 @@ optional<int> ptree::get_child_index(const std::string& key) const
     for(int i=0;i<size();i++)
 	if ((*this)[i].first == key)
 	    return i;
-    return boost::none;
+    return {};
 }
 
 int ptree::count(const std::string& key) const
@@ -56,22 +57,22 @@ bool ptree::operator!=(const ptree& p2) const
     return not ((*this) == p2);
 }
 
-boost::optional<ptree&>
+ptree*
 ptree::get_child_optional(const std::string& key)
 {
     if (auto index = get_child_index(key))
-	return (*this)[*index].second;
+	return &(*this)[*index].second;
     else
-	return boost::none;
+	return {};
 }
 
-boost::optional<const ptree&>
+const ptree*
 ptree::get_child_optional(const std::string& key) const
 {
     if (auto index = get_child_index(key))
-	return (*this)[*index].second;
+	return &(*this)[*index].second;
     else
-	return boost::none;
+	return {};
 }
 
 ptree& ptree::get_child(const std::string& key)
@@ -92,31 +93,31 @@ const ptree& ptree::get_child(const std::string& key) const
 	throw myexception()<<"No child with key '"<<key<<"'";
 }
 
-optional<ptree&> ptree::get_path_optional(const vector<string>& path, int i)
+ptree* ptree::get_path_optional(const vector<string>& path, int i)
 {
     assert(i <= path.size());
 
     // If the path is empty this is a reference to the current object
-    if (i == path.size()) return *this;
+    if (i == path.size()) return this;
 
     if (auto child = get_child_optional(path[i]))
 	return child->get_path_optional(path, i+1);
     else
-	return boost::none;
+	return {};
 }
 
 
-optional<const ptree&> ptree::get_path_optional(const vector<string>& path, int i) const
+const ptree* ptree::get_path_optional(const vector<string>& path, int i) const
 {
     assert(i <= path.size());
 
     // If the path is empty this is a reference to the current object
-    if (i == path.size()) return *this;
+    if (i == path.size()) return this;
 
     if (auto child = get_child_optional(path[i]))
 	return child->get_path_optional(path, i+1);
     else
-	return boost::none;
+	return {};
 }
 
 ptree& ptree::get_path(const vector<string>& path, int i)
